@@ -19,11 +19,11 @@ from typing import Dict, Optional
 
 import torch
 
-from nemo.collections.asr.parts.utils.nmesc_clustering import get_argmin_mat
+from nemo.collections.asr.parts.utils.offline_clustering import get_argmin_mat
 from nemo.collections.asr.parts.utils.speaker_utils import convert_rttm_line, prepare_split_data
 from nemo.collections.common.parts.preprocessing.collections import DiarizationSpeechLabel
 from nemo.core.classes import Dataset
-from nemo.core.neural_types import AudioSignal, EncodedRepresentation, LengthsType, NeuralType
+from nemo.core.neural_types import AudioSignal, EncodedRepresentation, LengthsType, NeuralType, ProbsType
 
 
 def get_scale_mapping_list(uniq_timestamps):
@@ -42,21 +42,22 @@ def get_scale_mapping_list(uniq_timestamps):
             The element at the m-th row and the n-th column of the scale mapping matrix indicates the (m+1)-th scale
             segment index which has the closest center distance with (n+1)-th segment in the base scale.
 
-            Example:
-                scale_mapping_argmat[2][101] = 85
+            - Example:
+                `scale_mapping_argmat[2][101] = 85`
 
-            In the above example, it means that 86-th segment in the 3rd scale (python index is 2) is mapped with
-            102-th segment in the base scale. Thus, the longer segments bound to have more repeating numbers since
-            multiple base scale segments (since the base scale has the shortest length) fall into the range of the
-            longer segments. At the same time, each row contains N numbers of indices where N is number of
-            segments in the base-scale (i.e., the finest scale).
+            In the above example, the code snippet means that 86-th segment in the 3rd scale (python index is 2) is
+            mapped to the 102-th segment in the base scale. Thus, the longer segments bound to have more repeating
+            numbers since multiple base scale segments (since the base scale has the shortest length) fall into the
+            range of the longer segments. At the same time, each row contains N numbers of indices where N is number
+            of segments in the base-scale (i.e., the finest scale).
     """
-    uniq_scale_dict = uniq_timestamps['scale_dict']
-    scale_mapping_argmat = [[] for _ in range(len(uniq_scale_dict.keys()))]
-
-    session_scale_mapping_dict = get_argmin_mat(uniq_scale_dict)
-    for scale_idx in sorted(uniq_scale_dict.keys()):
-        scale_mapping_argmat[scale_idx] = session_scale_mapping_dict[scale_idx]
+    timestamps_in_scales = []
+    for key, val in uniq_timestamps['scale_dict'].items():
+        timestamps_in_scales.append(torch.tensor(val['time_stamps']))
+    session_scale_mapping_list = get_argmin_mat(timestamps_in_scales)
+    scale_mapping_argmat = [[] for _ in range(len(uniq_timestamps['scale_dict'].keys()))]
+    for scale_idx in range(len(session_scale_mapping_list)):
+        scale_mapping_argmat[scale_idx] = session_scale_mapping_list[scale_idx]
     scale_mapping_argmat = torch.stack(scale_mapping_argmat)
     return scale_mapping_argmat
 
@@ -240,6 +241,7 @@ class _AudioMSDDTrainDataset(Dataset):
                 Unique sample ID for training.
             base_scale_clus_label (torch.tensor):
                 Tensor variable containing the speaker labels for the base-scale segments.
+        
         Returns:
             per_scale_clus_label (torch.tensor):
                 Tensor variable containing the speaker labels for each segment in each scale.
@@ -290,9 +292,7 @@ class _AudioMSDDTrainDataset(Dataset):
         seg_target_list, base_clus_label = [], []
         self.scale_n = len(self.multiscale_timestamp_dict[uniq_id]['scale_dict'])
         subseg_time_stamp_list = self.multiscale_timestamp_dict[uniq_id]["scale_dict"][self.scale_n - 1]["time_stamps"]
-        for line in subseg_time_stamp_list:
-            line_split = line.split()
-            seg_stt, seg_end = float(line_split[0]), float(line_split[1])
+        for (seg_stt, seg_end) in subseg_time_stamp_list:
             seg_stt_fr, seg_end_fr = int(seg_stt * self.frame_per_sec), int(seg_end * self.frame_per_sec)
             soft_label_vec_sess = torch.sum(fr_level_target[seg_stt_fr:seg_end_fr, :], axis=0) / (
                 seg_end_fr - seg_stt_fr
@@ -388,9 +388,9 @@ class _AudioMSDDTrainDataset(Dataset):
         ms_seg_counts = [0 for _ in range(self.scale_n)]
         for scale_idx in range(self.scale_n):
             scale_ts_list = []
-            for k, line in enumerate(self.multiscale_timestamp_dict[uniq_id]["scale_dict"][scale_idx]["time_stamps"]):
-                line_split = line.split()
-                seg_stt, seg_end = float(line_split[0]), float(line_split[1])
+            for k, (seg_stt, seg_end) in enumerate(
+                self.multiscale_timestamp_dict[uniq_id]["scale_dict"][scale_idx]["time_stamps"]
+            ):
                 stt, end = (
                     int((seg_stt - sample.offset) * self.frame_per_sec),
                     int((seg_end - sample.offset) * self.frame_per_sec),
