@@ -18,6 +18,7 @@ from typing import Any, Dict, List, Optional
 from omegaconf import MISSING
 
 import nemo.core.classes.dataset
+from nemo.collections.asr.metrics.wer import CTCDecodingConfig
 from nemo.collections.asr.modules.audio_preprocessing import (
     AudioToMelSpectrogramPreprocessorConfig,
     SpectrogramAugmentationConfig,
@@ -59,6 +60,7 @@ class ASRDatasetConfig(nemo.core.classes.dataset.DatasetConfig):
     # bucketing params
     bucketing_strategy: str = "synced_randomized"
     bucketing_batch_size: Optional[Any] = None
+    bucketing_weights: Optional[List[int]] = None
 
 
 @dataclass
@@ -83,8 +85,26 @@ class EncDecCTCConfig(model_cfg.ModelConfig):
     spec_augment: Optional[SpectrogramAugmentationConfig] = SpectrogramAugmentationConfig()
     encoder: ConvASREncoderConfig = ConvASREncoderConfig()
     decoder: ConvASRDecoderConfig = ConvASRDecoderConfig()
+    decoding: CTCDecodingConfig = CTCDecodingConfig()
 
 
 @dataclass
 class EncDecCTCModelConfig(model_cfg.NemoConfig):
     model: EncDecCTCConfig = EncDecCTCConfig()
+
+
+@dataclass
+class CacheAwareStreamingConfig:
+    chunk_size: int = 0  # the size of each chunk at each step, it can be a list of two integers to specify different chunk sizes for the first step and others
+    shift_size: int = 0  # the size of the shift in each step, it can be a list of two integers to specify different shift sizes for the first step and others
+
+    cache_drop_size: int = 0  # the number of steps to drop from the cache
+    last_channel_cache_size: int = 0  # the size of the needed cache for last channel layers
+
+    valid_out_len: int = 0  # the number of the steps in the final output which are valid (have the same value as in the offline mode)
+
+    pre_encode_cache_size: int = 0  # the size of the needed cache for the pre-encoding part of the model to avoid caching inside the pre-encoding layers
+    drop_extra_pre_encoded: int = 0  # the number of steps to get dropped after the pre-encoding layer
+
+    last_channel_num: int = 0  # number of the last channel layers (like MHA layers) which need caching in the model
+    last_time_num: int = 0  # number of the last time layers (like convolutions) which need caching in the model
