@@ -25,8 +25,11 @@ from pytorch_lightning import Trainer
 from torchmetrics import Accuracy
 from tqdm import tqdm
 
-from nemo.collections.asr.data.audio_to_label import AudioToSpeechLabelDataset
-from nemo.collections.asr.data.audio_to_label_dataset import get_tarred_speech_label_dataset
+from nemo.collections.asr.data.audio_to_label import AudioToSpeechLabelDataset, cache_datastore_manifests
+from nemo.collections.asr.data.audio_to_label_dataset import (
+    get_concat_tarred_speech_label_dataset,
+    get_tarred_speech_label_dataset,
+)
 from nemo.collections.asr.data.audio_to_text_dataset import convert_to_config_list
 from nemo.collections.asr.models.asr_model import ExportableEncDecModel
 from nemo.collections.asr.parts.preprocessing.features import WaveformFeaturizer
@@ -63,21 +66,21 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
 
         model = PretrainedModelInfo(
             pretrained_model_name="speakerverification_speakernet",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/speakerverification_speakernet/versions/1.0.0rc1/files/speakerverification_speakernet.nemo",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/speakerverification_speakernet/versions/1.16.0/files/speakerverification_speakernet.nemo",
             description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:speakerverification_speakernet",
         )
         result.append(model)
 
         model = PretrainedModelInfo(
             pretrained_model_name="ecapa_tdnn",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/ecapa_tdnn/versions/v1/files/ecapa_tdnn.nemo",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/ecapa_tdnn/versions/1.16.0/files/ecapa_tdnn.nemo",
             description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:ecapa_tdnn",
         )
         result.append(model)
 
         model = PretrainedModelInfo(
             pretrained_model_name="titanet_large",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/titanet_large/versions/v0/files/titanet-l.nemo",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/titanet_large/versions/v1/files/titanet-l.nemo",
             description="For details about this model, please visit https://catalog.ngc.nvidia.com/orgs/nvidia/teams/nemo/models/titanet_large",
         )
         result.append(model)
@@ -95,11 +98,9 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         self.world_size = 1
         self.cal_labels_occurrence_train = False
         self.labels_occurrence = None
+        self.labels = None
 
-        if 'num_classes' in cfg.decoder:
-            num_classes = cfg.decoder.num_classes
-        else:
-            num_classes = cfg.decoder.params.num_classes  # to pass test
+        num_classes = cfg.decoder.num_classes
 
         if 'loss' in cfg:
             if 'weight' in cfg.loss:
@@ -121,21 +122,6 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             weight = [sum(self.labels_occurrence) / (len(self.labels_occurrence) * i) for i in self.labels_occurrence]
 
         if 'loss' in cfg:
-            # To support older version checkpoints
-            if '_target_' not in cfg.loss:
-                logging.info(
-                    "Setting angular: true/false in decoder is deprecated and will be removed in 1.13 version, use specific loss with _target_"
-                )
-                OmegaConf.set_struct(cfg, True)
-                with open_dict(cfg):
-                    if 'angular' in cfg.decoder and cfg.decoder.angular:
-                        cfg.loss._target_ = "nemo.collections.asr.losses.angularloss.AngularSoftmaxLoss"
-                    else:
-                        # in case if specified angular=False but loss contained 'scale' or 'margin'
-                        cfg.loss.pop('scale', None)
-                        cfg.loss.pop('margin', None)
-                        cfg.loss._target_ = "nemo.collections.common.losses.cross_entropy.CrossEntropyLoss"
-
             cfg_eval_loss = copy.deepcopy(cfg.loss)
 
             if 'angular' in cfg.loss._target_:
@@ -165,9 +151,8 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         self.encoder = EncDecSpeakerLabelModel.from_config_dict(cfg.encoder)
         self.decoder = EncDecSpeakerLabelModel.from_config_dict(cfg.decoder)
 
-        self._macro_accuracy = Accuracy(num_classes=num_classes, average='macro')
+        self._macro_accuracy = Accuracy(num_classes=num_classes, top_k=1, average='macro', task='multiclass')
 
-        self.labels = None
         if hasattr(self._cfg, 'spec_augment') and self._cfg.spec_augment is not None:
             self.spec_augmentation = EncDecSpeakerLabelModel.from_config_dict(self._cfg.spec_augment)
         else:
@@ -183,6 +168,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         manifest_filepaths = convert_to_config_list(data_layer_config['manifest_filepath'])
 
         for manifest_filepath in itertools.chain.from_iterable(manifest_filepaths):
+            cache_datastore_manifests(manifest_filepaths=manifest_filepath)
             collection = ASRSpeechLabel(
                 manifests_files=manifest_filepath,
                 min_duration=data_layer_config.get("min_duration", None),
@@ -215,13 +201,22 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
                 return None
 
             shuffle_n = config.get('shuffle_n', 4 * config['batch_size']) if shuffle else 0
-            dataset = get_tarred_speech_label_dataset(
-                featurizer=featurizer,
-                config=config,
-                shuffle_n=shuffle_n,
-                global_rank=self.global_rank,
-                world_size=self.world_size,
-            )
+            if config.get("is_concat", False):
+                dataset = get_concat_tarred_speech_label_dataset(
+                    featurizer=featurizer,
+                    config=config,
+                    shuffle_n=shuffle_n,
+                    global_rank=self.global_rank,
+                    world_size=self.world_size,
+                )
+            else:
+                dataset = get_tarred_speech_label_dataset(
+                    featurizer=featurizer,
+                    config=config,
+                    shuffle_n=shuffle_n,
+                    global_rank=self.global_rank,
+                    world_size=self.world_size,
+                )
             shuffle = False
         else:
             if 'manifest_filepath' in config and config['manifest_filepath'] is None:
@@ -354,7 +349,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         top_k = self._accuracy.compute()
         self._accuracy.reset()
         for i, top_i in enumerate(top_k):
-            self.log(f'training_batch_accuracy_top@{i}', top_i)
+            self.log(f'training_batch_accuracy_top_{i}', top_i)
 
         return {'loss': loss}
 
@@ -365,7 +360,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         acc_top_k = self._accuracy(logits=logits, labels=labels)
         correct_counts, total_counts = self._accuracy.correct_counts_k, self._accuracy.total_counts_k
         self._macro_accuracy.update(preds=logits, target=labels)
-        stats = self._macro_accuracy._get_final_stats()
+        stats = self._macro_accuracy._final_state()
 
         return {
             f'{tag}_loss': loss_value,
@@ -395,7 +390,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
 
         self.log(f'{tag}_loss', loss_mean, sync_dist=True)
         for top_k, score in zip(self._accuracy.top_k, topk_scores):
-            self.log(f'{tag}_acc_micro_top@{top_k}', score, sync_dist=True)
+            self.log(f'{tag}_acc_micro_top_{top_k}', score, sync_dist=True)
         self.log(f'{tag}_acc_macro', macro_accuracy_score, sync_dist=True)
 
         return {
@@ -417,20 +412,19 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         return self.multi_evaluation_epoch_end(outputs, dataloader_idx, 'test')
 
     @torch.no_grad()
-    def get_embedding(self, path2audio_file):
+    def infer_file(self, path2audio_file):
         """
-        Returns the speaker embeddings for a provided audio file.
-
         Args:
-            path2audio_file: path to audio wav file
+            path2audio_file: path to an audio wav file
 
         Returns:
-            embs: speaker embeddings
+            emb: speaker embeddings (Audio representations)
+            logits: logits corresponding of final layer
         """
         audio, sr = librosa.load(path2audio_file, sr=None)
         target_sr = self._cfg.train_ds.get('sample_rate', 16000)
         if sr != target_sr:
-            audio = librosa.core.resample(audio, sr, target_sr)
+            audio = librosa.core.resample(audio, orig_sr=sr, target_sr=target_sr)
         audio_length = audio.shape[0]
         device = self.device
         audio = np.array(audio)
@@ -441,13 +435,50 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         mode = self.training
         self.freeze()
 
-        _, embs = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
+        logits, emb = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
 
         self.train(mode=mode)
         if mode is True:
             self.unfreeze()
         del audio_signal, audio_signal_len
-        return embs
+        return emb, logits
+
+    def get_label(self, path2audio_file):
+        """
+        Returns label of path2audio_file from classes the model was trained on.
+        Args:
+            path2audio_file: path to audio wav file
+
+        Returns:
+            label: label corresponding to the trained model
+        """
+        _, logits = self.infer_file(path2audio_file=path2audio_file)
+
+        trained_labels = self._cfg['train_ds'].get('labels', None)
+        if trained_labels is not None:
+            trained_labels = list(trained_labels)
+            label_id = logits.argmax(axis=1)
+            label = trained_labels[int(label_id[0])]
+        else:
+            logging.info("labels are not saved to model, hence only outputting the label id index")
+            label = logits.argmax(axis=1)
+
+        return label
+
+    def get_embedding(self, path2audio_file):
+        """
+        Returns the speaker embeddings for a provided audio file.
+
+        Args:
+            path2audio_file: path to an audio wav file
+
+        Returns:
+            emb: speaker embeddings (Audio representations)
+        """
+
+        emb, _ = self.infer_file(path2audio_file=path2audio_file)
+
+        return emb
 
     @torch.no_grad()
     def verify_speakers(self, path2audio_file1, path2audio_file2, threshold=0.7):
@@ -478,35 +509,66 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             logging.info(" two audio files are from different speakers")
             return False
 
-    @staticmethod
     @torch.no_grad()
-    def get_batch_embeddings(speaker_model, manifest_filepath, batch_size=32, sample_rate=16000, device='cuda'):
+    def batch_inference(self, manifest_filepath, batch_size=32, sample_rate=16000, device='cuda'):
+        """
+        Perform batch inference on EncDecSpeakerLabelModel.
+        To perform inference on single audio file, once can use infer_model, get_label or get_embedding
 
-        speaker_model.eval()
-        if device == 'cuda':
-            speaker_model.to(device)
+        To map predicted labels, one can do
+            `arg_values = logits.argmax(axis=1)`
+            `pred_labels = list(map(lambda t : trained_labels[t], arg_values))`
+
+        Args:
+            manifest_filepath: Path to manifest file
+            batch_size: batch size to perform batch inference
+            sample_rate: sample rate of audio files in manifest file
+            device: compute device to perform operations.
+
+        Returns:
+            The variables below all follow the audio file order in the manifest file.
+            embs: embeddings of files provided in manifest file
+            logits: logits of final layer of EncDecSpeakerLabel Model
+            gt_labels: labels from manifest file (needed for speaker enrollment and testing)
+            trained_labels: Classification labels sorted in the order that they are mapped by the trained model
+
+        """
+        mode = self.training
+        self.freeze()
+        self.eval()
+        self.to(device)
+        trained_labels = self._cfg['train_ds']['labels']
+        if trained_labels is not None:
+            trained_labels = list(trained_labels)
 
         featurizer = WaveformFeaturizer(sample_rate=sample_rate)
+
         dataset = AudioToSpeechLabelDataset(manifest_filepath=manifest_filepath, labels=None, featurizer=featurizer)
 
         dataloader = torch.utils.data.DataLoader(
             dataset=dataset, batch_size=batch_size, collate_fn=dataset.fixed_seq_collate_fn,
         )
 
-        all_logits = []
-        all_labels = []
-        all_embs = []
+        logits = []
+        embs = []
+        gt_labels = []
 
         for test_batch in tqdm(dataloader):
             if device == 'cuda':
                 test_batch = [x.to(device) for x in test_batch]
             audio_signal, audio_signal_len, labels, _ = test_batch
-            logits, embs = speaker_model.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
+            logit, emb = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
 
-            all_logits.extend(logits.cpu().numpy())
-            all_labels.extend(labels.cpu().numpy())
-            all_embs.extend(embs.cpu().numpy())
+            logits.extend(logit.cpu().numpy())
+            gt_labels.extend(labels.cpu().numpy())
+            embs.extend(emb.cpu().numpy())
 
-        all_logits, true_labels, all_embs = np.asarray(all_logits), np.asarray(all_labels), np.asarray(all_embs)
+        gt_labels = list(map(lambda t: dataset.id2label[t], gt_labels))
 
-        return all_embs, all_logits, true_labels, dataset.id2label
+        self.train(mode=mode)
+        if mode is True:
+            self.unfreeze()
+
+        logits, embs, gt_labels = np.asarray(logits), np.asarray(embs), np.asarray(gt_labels)
+
+        return embs, logits, gt_labels, trained_labels
