@@ -16,6 +16,7 @@ import copy
 import json
 import os
 import pickle as pkl
+import tempfile
 from collections import OrderedDict
 from statistics import mode
 from typing import Any, Dict, List, Optional, Tuple, Union
@@ -24,13 +25,22 @@ import numpy as np
 import torch
 from hydra.utils import instantiate
 from omegaconf import DictConfig, open_dict
+from pyannote.metrics.diarization import DiarizationErrorRate
 from pytorch_lightning import Trainer
+from pytorch_lightning.utilities import rank_zero_only
 from tqdm import tqdm
 
 from nemo.collections.asr.data.audio_to_diar_label import AudioToSpeechMSDDInferDataset, AudioToSpeechMSDDTrainDataset
+from nemo.collections.asr.metrics.der import score_labels
 from nemo.collections.asr.metrics.multi_binary_acc import MultiBinaryAccuracy
 from nemo.collections.asr.models import ClusteringDiarizer
 from nemo.collections.asr.models.asr_model import ExportableEncDecModel
+from nemo.collections.asr.models.clustering_diarizer import (
+    _MODEL_CONFIG_YAML,
+    _SPEAKER_MODEL,
+    _VAD_MODEL,
+    get_available_model_names,
+)
 from nemo.collections.asr.models.label_models import EncDecSpeakerLabelModel
 from nemo.collections.asr.parts.preprocessing.features import WaveformFeaturizer
 from nemo.collections.asr.parts.utils.speaker_utils import (
@@ -41,7 +51,6 @@ from nemo.collections.asr.parts.utils.speaker_utils import (
     get_uniq_id_list_from_manifest,
     make_rttm_with_overlap,
     parse_scale_configs,
-    score_labels,
 )
 from nemo.core.classes import ModelPT
 from nemo.core.classes.common import PretrainedModelInfo, typecheck
@@ -59,7 +68,7 @@ except ImportError:
         yield
 
 
-__all__ = ['EncDecDiarLabelModel', 'ClusterEmbedding', 'OverlapAwareDiarizer']
+__all__ = ['EncDecDiarLabelModel', 'ClusterEmbedding', 'NeuralDiarizer']
 
 
 class EncDecDiarLabelModel(ModelPT, ExportableEncDecModel):
@@ -81,7 +90,15 @@ class EncDecDiarLabelModel(ModelPT, ExportableEncDecModel):
         Returns:
             List of available pre-trained models.
         """
-        return None
+        result = []
+
+        model = PretrainedModelInfo(
+            pretrained_model_name="diar_msdd_telephonic",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/diar_msdd_telephonic/versions/1.0.1/files/diar_msdd_telephonic.nemo",
+            description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:diar_msdd_telephonic",
+        )
+        result.append(model)
+        return result
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
         """
@@ -173,12 +190,15 @@ class EncDecDiarLabelModel(ModelPT, ExportableEncDecModel):
             self._speaker_model = EncDecSpeakerLabelModel.load_from_checkpoint(model_path, map_location=rank_id)
             logging.info("Speaker Model restored locally from {}".format(model_path))
         else:
-            if model_path not in EncDecSpeakerLabelModel.get_available_model_names():
+            if model_path not in get_available_model_names(EncDecSpeakerLabelModel):
                 logging.warning(
                     "requested {} model name not available in pretrained models, instead".format(model_path)
                 )
+                model_path = "titanet_large"
             logging.info("Loading pretrained {} model from NGC".format(model_path))
-            self._speaker_model = EncDecSpeakerLabelModel.from_pretrained(model_name=model_path)
+            self.msdd._speaker_model = EncDecSpeakerLabelModel.from_pretrained(
+                model_name=model_path, map_location=rank_id
+            )
         self._speaker_params = self.cfg_msdd_model.diarizer.speaker_embeddings.parameters
 
     def __setup_dataloader_from_config(self, config):
@@ -328,10 +348,10 @@ class EncDecDiarLabelModel(ModelPT, ExportableEncDecModel):
                 the multi-scale input matrix during forward propagating.
 
 		Example: `batch_size=3, scale_n=6, emb_dim=192`
-                    ms_seg_counts =
-                     [[8,  9, 12, 16, 25, 51],
-                      [11, 13, 14, 17, 25, 51],
-                      [ 9,  9, 11, 16, 23, 50]]
+                    ms_seg_counts =  
+                     [[8,  9, 12, 16, 25, 51],  
+                      [11, 13, 14, 17, 25, 51],  
+                      [ 9,  9, 11, 16, 23, 50]]  
 
 		In this function, `ms_seg_counts` is used to get the actual length of each embedding sequence without
 		zero-padding.
@@ -375,13 +395,13 @@ class EncDecDiarLabelModel(ModelPT, ExportableEncDecModel):
                 multi-scale input tensors during forward propagating.
 
                 Example: `batch_size=3, scale_n=6, emb_dim=192`
-                    ms_seg_counts =
-                     [[8,  9, 12, 16, 25, 51],
-                      [11, 13, 14, 17, 25, 51],
-                      [ 9,  9, 11, 16, 23, 50]]
-                    Counts of merged segments: (121, 131, 118)
-                    embs has shape of (370, 192)
-                    clus_label_index has shape of (3, 131)
+                    ms_seg_counts =  
+                     [[8,  9, 12, 16, 25, 51],  
+                      [11, 13, 14, 17, 25, 51],  
+                      [ 9,  9, 11, 16, 23, 50]]  
+                    Counts of merged segments: (121, 131, 118)  
+                    embs has shape of (370, 192)  
+                    clus_label_index has shape of (3, 131)  
 
                 Shape: (batch_size, scale_n)
 
@@ -439,31 +459,31 @@ class EncDecDiarLabelModel(ModelPT, ExportableEncDecModel):
         `self.emb_batch_size` is 0.
 
         Args:
-            processed_signal: (Tensor)
+            processed_signal (Tensor):
                 Zero-padded Feature input.
                 Shape: (batch_size, feat_dim, the longest feature sequence length)
-            processed_signal_len: (Tensor)
+            processed_signal_len (Tensor):
                 The actual legnth of feature input without zero-padding.
                 Shape: (batch_size,)
-            ms_seg_timestamps: (Tensor)
+            ms_seg_timestamps (Tensor):
                 Timestamps of the base-scale segments.
                 Shape: (batch_size, scale_n, number of base-scale segments, self.num_spks_per_model)
-            ms_seg_counts: (Tensor)
+            ms_seg_counts (Tensor):
                 Cumulative sum of the number of segments in each scale. This information is needed to reconstruct
                 the multi-scale input matrix during forward propagating.
                 Shape: (batch_size, scale_n)
 
         Returns:
-            ms_mel_feat: (Tensor)
+            ms_mel_feat (Tensor):
                 Feature input stream split into the same length.
                 Shape: (total number of segments, feat_dim, self.frame_per_sec * the-longest-scale-length)
-            ms_mel_feat_len: (Tensor)
+            ms_mel_feat_len (Tensor):
                 The actual length of feature without zero-padding.
                 Shape: (total number of segments,)
-            seq_len: (Tensor)
+            seq_len (Tensor):
                 The length of the input embedding sequences.
                 Shape: (total number of segments,)
-            detach_ids: (tuple)
+            detach_ids (tuple):
                 Tuple containing both detached embeding indices and attached embedding indices
         """
         device = processed_signal.device
@@ -642,14 +662,8 @@ class ClusterEmbedding:
             Config dictionary from diarization inference YAML file
         cfg_msdd_model (DictConfig):
             Config dictionary from MSDD model checkpoint file
-        self.clus_diar_model (class `ClusteringDiarizer`):
-            This is a placeholder for class instance of `ClusteringDiarizer`
-        self.msdd_model (class `EncDecDiarLabelModel`):
-            This is a placeholder for class instance of `EncDecDiarLabelModel`
-        self.spk_emb_state_dict (dict):
-            Dictionary containing `state_dict` of speaker embedding model
-        self.use_speaker_model_from_ckpt (bool):
-            If True, speaker embedding model included in MSDD checkpoint is used.
+        self._speaker_model (class `EncDecSpeakerLabelModel`):
+            This is a placeholder for class instance of `EncDecSpeakerLabelModel`
         self.scale_window_length_list (list):
             List containing the window lengths (i.e., scale length) of each scale.
         self.scale_n (int):
@@ -662,11 +676,7 @@ class ClusterEmbedding:
         self.cfg_diar_infer = cfg_diar_infer
         self._cfg_msdd = cfg_msdd_model
         self.clus_diar_model = None
-        self.msdd_model = None
-        self.spk_emb_state_dict = {}
-        self.use_speaker_model_from_ckpt = self.cfg_diar_infer.diarizer.msdd_model.parameters.get(
-            'use_speaker_model_from_ckpt', True
-        )
+        self._speaker_model = None
         self.scale_window_length_list = list(
             self.cfg_diar_infer.diarizer.speaker_embeddings.parameters.window_length_in_sec
         )
@@ -828,15 +838,11 @@ class ClusterEmbedding:
         self.cfg_diar_infer.diarizer.out_dir = emb_dir
 
         # Run ClusteringDiarizer which includes system VAD or oracle VAD.
-        self.clus_diar_model = ClusteringDiarizer(cfg=self.cfg_diar_infer)
+        self.clus_diar_model = ClusteringDiarizer(cfg=self.cfg_diar_infer, speaker_model=self._speaker_model)
         self._out_dir = self.clus_diar_model._diarizer_params.out_dir
         self.out_rttm_dir = os.path.join(self._out_dir, 'pred_ovl_rttms')
         os.makedirs(self.out_rttm_dir, exist_ok=True)
 
-        # Load speaker embedding model state_dict which is loaded from the MSDD checkpoint.
-        if self.use_speaker_model_from_ckpt:
-            self.clus_diar_model._speaker_model.load_state_dict(self.spk_emb_state_dict)
-
         self.clus_diar_model._cluster_params = self.cfg_diar_infer.diarizer.clustering.parameters
         self.clus_diar_model.multiscale_args_dict[
             "multiscale_weights"
@@ -852,7 +858,7 @@ class ClusterEmbedding:
 
         # If RTTM (ground-truth diarization annotation) files do not exist, scores is None.
         if scores is not None:
-            metric, speaker_mapping_dict = scores
+            metric, speaker_mapping_dict, _ = scores
         else:
             metric, speaker_mapping_dict = None, None
 
@@ -951,7 +957,7 @@ class ClusterEmbedding:
         return emb_scale_seq_dict
 
 
-class OverlapAwareDiarizer:
+class NeuralDiarizer:
     """
     Class for inference based on multiscale diarization decoder (MSDD). MSDD requires initializing clustering results from
     clustering diarizer. Overlap-aware diarizer requires separate RTTM generation and evaluation modules to check the effect of
@@ -962,6 +968,17 @@ class OverlapAwareDiarizer:
         """ """
         self._cfg = cfg
 
+        # Parameter settings for MSDD model
+        self.use_speaker_model_from_ckpt = cfg.diarizer.msdd_model.parameters.get('use_speaker_model_from_ckpt', True)
+        self.use_clus_as_main = cfg.diarizer.msdd_model.parameters.get('use_clus_as_main', False)
+        self.max_overlap_spks = cfg.diarizer.msdd_model.parameters.get('max_overlap_spks', 2)
+        self.num_spks_per_model = cfg.diarizer.msdd_model.parameters.get('num_spks_per_model', 2)
+        self.use_adaptive_thres = cfg.diarizer.msdd_model.parameters.get('use_adaptive_thres', True)
+        self.max_pred_length = cfg.diarizer.msdd_model.parameters.get('max_pred_length', 0)
+        self.diar_eval_settings = cfg.diarizer.msdd_model.parameters.get(
+            'diar_eval_settings', [(0.25, True), (0.25, False), (0.0, False)]
+        )
+
         self._init_msdd_model(cfg)
         self.diar_window_length = cfg.diarizer.msdd_model.parameters.diar_window_length
         self.msdd_model.cfg = self.transfer_diar_params_to_model_params(self.msdd_model, cfg)
@@ -970,22 +987,13 @@ class OverlapAwareDiarizer:
 
         # Initialize clustering and embedding preparation instance (as a diarization encoder).
         self.clustering_embedding = ClusterEmbedding(cfg_diar_infer=cfg, cfg_msdd_model=self.msdd_model.cfg)
-        self.clustering_embedding.spk_emb_state_dict = self.spk_emb_state_dict
+        self.clustering_embedding._speaker_model = self._speaker_model
 
         # Parameters for creating diarization results from MSDD outputs.
         self.clustering_max_spks = self.msdd_model._cfg.max_num_of_spks
-
         self.overlap_infer_spk_limit = cfg.diarizer.msdd_model.parameters.get(
             'overlap_infer_spk_limit', self.clustering_max_spks
         )
-        self.use_clus_as_main = cfg.diarizer.msdd_model.parameters.get('use_clus_as_main', False)
-        self.max_overlap_spks = cfg.diarizer.msdd_model.parameters.get('max_overlap_spks', 2)
-        self.num_spks_per_model = cfg.diarizer.msdd_model.parameters.get('num_spks_per_model', 2)
-        self.use_adaptive_thres = cfg.diarizer.msdd_model.parameters.get('use_adaptive_thres', True)
-        self.max_pred_length = cfg.diarizer.msdd_model.parameters.get('max_pred_length', 0)
-        self.diar_eval_settings = cfg.diarizer.msdd_model.parameters.get(
-            'diar_eval_settings', [(0.25, True), (0.25, False), (0.0, False)]
-        )
 
     def transfer_diar_params_to_model_params(self, msdd_model, cfg):
         """
@@ -1000,7 +1008,36 @@ class OverlapAwareDiarizer:
         msdd_model._cfg.max_num_of_spks = cfg.diarizer.clustering.parameters.max_num_speakers
         return msdd_model.cfg
 
-    def extract_standalone_speaker_model(self, ext: str = '.ckpt') -> Dict:
+    @rank_zero_only
+    def save_to(self, save_path: str):
+        """
+        Saves model instances (weights and configuration) into EFF archive.
+        You can use "restore_from" method to fully restore instance from .nemo file.
+
+        .nemo file is an archive (tar.gz) with the following:
+            model_config.yaml - model configuration in .yaml format. You can deserialize this into cfg argument for model's constructor
+            model_wights.chpt - model checkpoint
+
+        Args:
+            save_path: Path to .nemo file where model instance should be saved
+        """
+        self.clus_diar = self.clustering_embedding.clus_diar_model
+        _NEURAL_DIAR_MODEL = "msdd_model.nemo"
+
+        with tempfile.TemporaryDirectory() as tmpdir:
+            config_yaml = os.path.join(tmpdir, _MODEL_CONFIG_YAML)
+            spkr_model = os.path.join(tmpdir, _SPEAKER_MODEL)
+            neural_diar_model = os.path.join(tmpdir, _NEURAL_DIAR_MODEL)
+
+            self.clus_diar.to_config_file(path2yaml_file=config_yaml)
+            if self.clus_diar.has_vad_model:
+                vad_model = os.path.join(tmpdir, _VAD_MODEL)
+                self.clus_diar._vad_model.save_to(vad_model)
+            self.clus_diar._speaker_model.save_to(spkr_model)
+            self.msdd_model.save_to(neural_diar_model)
+            self.clus_diar.__make_nemo_file_from_folder(filename=save_path, source_dir=tmpdir)
+
+    def extract_standalone_speaker_model(self, prefix: str = 'msdd._speaker_model.') -> Dict:
         """
         MSDD model file contains speaker embedding model and MSDD model. This function extracts standalone speaker model and save it to
         `self.spk_emb_state_dict` to be loaded separately for clustering diarizer.
@@ -1012,33 +1049,43 @@ class OverlapAwareDiarizer:
             standalone_model_path (str):
                 Path to the extracted standalone model without speaker embedding extractor model.
         """
-        loaded_model = torch.load(self._cfg.diarizer.msdd_model.model_path)
-        spk_emb_state_dict = {}
+        model_state_dict = self.msdd_model.state_dict()
         spk_emb_module_names = []
-        for name in loaded_model['state_dict'].keys():
-            if 'msdd._speaker_model' in name:
+        for name in model_state_dict.keys():
+            if prefix in name:
                 spk_emb_module_names.append(name)
 
+        spk_emb_state_dict = {}
         for name in spk_emb_module_names:
-            org_name = name.replace('msdd._speaker_model.', '')
-            spk_emb_state_dict[org_name] = loaded_model['state_dict'][name]
+            org_name = name.replace(prefix, '')
+            spk_emb_state_dict[org_name] = model_state_dict[name]
 
-        return spk_emb_state_dict
+        _speaker_model = EncDecSpeakerLabelModel.from_config_dict(self.msdd_model.cfg.speaker_model_cfg)
+        _speaker_model.load_state_dict(spk_emb_state_dict)
+        return _speaker_model
 
     def _init_msdd_model(self, cfg: DictConfig):
         """
         Initialized MSDD model with the provided config. Load either from `.nemo` file or `.ckpt` checkpoint files.
         """
-        if cfg.diarizer.msdd_model.model_path.endswith('.nemo'):
-            logging.info(f"Using local nemo file from {cfg.diarizer.msdd_model.model_path}")
-            self.msdd_model = EncDecDiarLabelModel.restore_from(restore_path=cfg.diarizer.msdd_model.model_path)
-            self.spk_emb_state_dict = self.extract_standalone_speaker_model('.nemo')
-        elif cfg.diarizer.msdd_model.model_path.endswith('.ckpt'):
-            logging.info(f"Using local checkpoint from {cfg.diarizer.msdd_model.model_path}")
-            self.msdd_model = EncDecDiarLabelModel.load_from_checkpoint(
-                checkpoint_path=cfg.diarizer.msdd_model.model_path
-            )
-            self.spk_emb_state_dict = self.extract_standalone_speaker_model('.ckpt')
+        model_path = cfg.diarizer.msdd_model.model_path
+        if model_path.endswith('.nemo'):
+            logging.info(f"Using local nemo file from {model_path}")
+            self.msdd_model = EncDecDiarLabelModel.restore_from(restore_path=model_path)
+        elif model_path.endswith('.ckpt'):
+            logging.info(f"Using local checkpoint from {model_path}")
+            self.msdd_model = EncDecDiarLabelModel.load_from_checkpoint(checkpoint_path=model_path)
+        else:
+            if model_path not in get_available_model_names(EncDecDiarLabelModel):
+                logging.warning(f"requested {model_path} model name not available in pretrained models, instead")
+            logging.info("Loading pretrained {} model from NGC".format(model_path))
+            self.msdd_model = EncDecDiarLabelModel.from_pretrained(model_name=model_path)
+
+        # Load speaker embedding model state_dict which is loaded from the MSDD checkpoint.
+        if self.use_speaker_model_from_ckpt:
+            self._speaker_model = self.extract_standalone_speaker_model()
+        else:
+            self._speaker_model = None
 
     def get_pred_mat(self, data_list: List[Union[Tuple[int], List[torch.Tensor]]]) -> torch.Tensor:
         """
@@ -1053,9 +1100,9 @@ class OverlapAwareDiarizer:
                     Examples: (0, 1, 2)
                 data[1]: Tensor containing estimaged sigmoid values.
                    [[0.0264, 0.9995],
-		    [0.0112, 1.0000],
-		    ...,
-		    [1.0000, 0.0512]]
+                    [0.0112, 1.0000],
+                    ...,
+                    [1.0000, 0.0512]]
 
         Returns:
             sum_pred (Tensor):
@@ -1113,7 +1160,7 @@ class OverlapAwareDiarizer:
         self.msdd_model.clus_test_label_dict = cluster_embeddings.clus_test_label_dict
         self.msdd_model.emb_seq_test = cluster_embeddings.emb_seq_test
 
-    def diarize(self):
+    def diarize(self) -> List[Optional[List[Tuple[DiarizationErrorRate, Dict]]]]:
         """
         Launch diarization pipeline which starts from VAD (or a oracle VAD stamp generation), initialization clustering and multiscale diarization decoder (MSDD).
         Note that the result of MSDD can include multiple speakers at the same time. Therefore, RTTM output of MSDD needs to be based on `make_rttm_with_overlap()`
@@ -1124,8 +1171,8 @@ class OverlapAwareDiarizer:
         self.msdd_model.pairwise_infer = True
         self.get_emb_clus_infer(self.clustering_embedding)
         preds_list, targets_list, signal_lengths_list = self.run_pairwise_diarization()
-        for threshold in list(self._cfg.diarizer.msdd_model.parameters.sigmoid_threshold):
-            self.run_overlap_aware_eval(preds_list, threshold)
+        thresholds = list(self._cfg.diarizer.msdd_model.parameters.sigmoid_threshold)
+        return [self.run_overlap_aware_eval(preds_list, threshold) for threshold in thresholds]
 
     def get_range_average(
         self, signals: torch.Tensor, emb_vectors: torch.Tensor, diar_window_index: int, test_data_collection: List[Any]
@@ -1199,7 +1246,7 @@ class OverlapAwareDiarizer:
         return emb_vectors_split, emb_seq, seq_len
 
     def get_range_clus_avg_emb(
-        self, test_batch: List[torch.Tensor], _test_data_collection: List[Any]
+        self, test_batch: List[torch.Tensor], _test_data_collection: List[Any], device: torch.device('cpu')
     ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
         """
         This function is only used when `get_range_average` function is called. This module calculates cluster-average embeddings for
@@ -1235,9 +1282,9 @@ class OverlapAwareDiarizer:
                 sess_emb_vectors.append(emb_vectors_split)
                 sess_emb_seq.append(emb_seq)
                 sess_sig_lengths.append(seq_len)
-        sess_emb_vectors = torch.stack(sess_emb_vectors)
-        sess_emb_seq = torch.stack(sess_emb_seq)
-        sess_sig_lengths = torch.tensor(sess_sig_lengths)
+        sess_emb_vectors = torch.stack(sess_emb_vectors).to(device)
+        sess_emb_seq = torch.stack(sess_emb_seq).to(device)
+        sess_sig_lengths = torch.tensor(sess_sig_lengths).to(device)
         return sess_emb_vectors, sess_emb_seq, sess_sig_lengths
 
     def diar_infer(
@@ -1267,7 +1314,7 @@ class OverlapAwareDiarizer:
         if self._cfg.diarizer.msdd_model.parameters.split_infer:
             split_count = torch.ceil(torch.tensor(signals.shape[1] / self.diar_window_length)).int()
             sess_emb_vectors, sess_emb_seq, sess_sig_lengths = self.get_range_clus_avg_emb(
-                test_batch, test_data_collection,
+                test_batch, test_data_collection, device=self.msdd_model.device
             )
             with autocast():
                 _preds, scale_weights = self.msdd_model.forward_infer(
@@ -1281,7 +1328,7 @@ class OverlapAwareDiarizer:
         else:
             with autocast():
                 _preds, scale_weights = self.msdd_model.forward_infer(
-                    input_signal=signals, input_signal_length=signal_lengths, emb_vectors=emb_vectors, targets=_targets
+                    input_signal=signals, input_signal_length=signal_lengths, emb_vectors=emb_vectors, targets=None
                 )
         self.max_pred_length = max(_preds.shape[1], self.max_pred_length)
         preds = torch.zeros(_preds.shape[0], self.max_pred_length, _preds.shape[2])
@@ -1329,7 +1376,9 @@ class OverlapAwareDiarizer:
         integrated_preds_list = self.get_integrated_preds_list(uniq_id_list, test_data_collection, preds_list)
         return integrated_preds_list, targets_list, signal_lengths_list
 
-    def run_overlap_aware_eval(self, preds_list: List[torch.Tensor], threshold: float):
+    def run_overlap_aware_eval(
+        self, preds_list: List[torch.Tensor], threshold: float
+    ) -> List[Optional[Tuple[DiarizationErrorRate, Dict]]]:
         """
         Based on the predicted sigmoid values, render RTTM files then evaluate the overlap-aware diarization results.
 
@@ -1344,20 +1393,23 @@ class OverlapAwareDiarizer:
         logging.info(
             f"     [Threshold: {threshold:.4f}] [use_clus_as_main={self.use_clus_as_main}] [diar_window={self.diar_window_length}]"
         )
+        outputs = []
         for k, (collar, ignore_overlap) in enumerate(self.diar_eval_settings):
             all_reference, all_hypothesis = make_rttm_with_overlap(
                 self.manifest_filepath,
                 self.msdd_model.clus_test_label_dict,
                 preds_list,
                 threshold=threshold,
-                infer_overlap=(not ignore_overlap),
+                infer_overlap=True,
                 use_clus_as_main=self.use_clus_as_main,
                 overlap_infer_spk_limit=self.overlap_infer_spk_limit,
                 use_adaptive_thres=self.use_adaptive_thres,
                 max_overlap_spks=self.max_overlap_spks,
                 out_rttm_dir=self.out_rttm_dir,
             )
-            score_labels(
+            output = score_labels(
                 self.AUDIO_RTTM_MAP, all_reference, all_hypothesis, collar=collar, ignore_overlap=ignore_overlap,
             )
+            outputs.append(output)
         logging.info(f"  \n")
+        return outputs
