@@ -14,7 +14,8 @@
 
 import math
 from collections import OrderedDict
-from typing import List, Optional
+from dataclasses import dataclass
+from typing import List, Optional, Set
 
 import torch
 import torch.distributed
@@ -26,11 +27,18 @@ from nemo.collections.asr.parts.mixins.streaming import StreamingEncoder
 from nemo.collections.asr.parts.submodules.causal_convs import CausalConv1D
 from nemo.collections.asr.parts.submodules.conformer_modules import ConformerLayer
 from nemo.collections.asr.parts.submodules.multi_head_attention import (
+    LocalAttRelPositionalEncoding,
     MultiHeadAttention,
     PositionalEncoding,
     RelPositionalEncoding,
+    RelPositionMultiHeadAttention,
+    RelPositionMultiHeadAttentionLongformer,
+)
+from nemo.collections.asr.parts.submodules.subsampling import (
+    ConvSubsampling,
+    StackingSubsampling,
+    SubsamplingReductionModule,
 )
-from nemo.collections.asr.parts.submodules.subsampling import ConvSubsampling, StackingSubsampling
 from nemo.collections.asr.parts.utils import adapter_utils
 from nemo.core.classes.common import typecheck
 from nemo.core.classes.exportable import Exportable
@@ -60,16 +68,27 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
             Defaults to 4.
         subsampling_conv_channels (int): the size of the convolutions in the subsampling module
             Defaults to -1 which would set it to d_model.
+        reduction (str, Optional): the method of reduction, choices=['pooling', 'striding']. If no value
+            is passed, then no reduction is performed and the models runs with the original 4x subsampling.
+        reduction_position (int, Optional): the index of the layer to apply reduction. If -1, apply reduction
+            at the end.
+        reduction_factor (int): the reduction factor which should be either 1 or a power of 2
+            Defaults to 1.
         ff_expansion_factor (int): the expansion factor in feed forward layers
             Defaults to 4.
         self_attention_model (str): type of the attention layer and positional encoding
             'rel_pos': relative positional embedding and Transformer-XL
+            'rel_pos_local_attn': relative positional embedding and Transformer-XL with local attention using
+                overlapping chunks. Attention context is determined by att_context_size parameter.
             'abs_pos': absolute positional embedding and Transformer
-            default is rel_pos.
+            Default is rel_pos.
         pos_emb_max_len (int): the maximum length of positional embeddings
-            Defaulst to 5000
+            Defaults to 5000
         n_heads (int): number of heads in multi-headed attention layers
             Defaults to 4.
+        att_context_size (List[int]): List of 2 ints corresponding to left and right attention context sizes,
+            or None for full context.
+            Defaults to None.
         xscaling (bool): enables scaling the inputs to the multi-headed attention layers by sqrt(d_model)
             Defaults to True.
         untie_biases (bool): whether to not share (untie) the bias weights between layers of Transformer-XL
@@ -80,6 +99,8 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
             Defaults to 'batch_norm'.
         dropout (float): the dropout rate used in all layers except the attention layers
             Defaults to 0.1.
+        dropout_pre_encoder (float): the dropout rate used before the encoder
+            Defaults to 0.1.
         dropout_emb (float): the dropout rate used for the positional embeddings
             Defaults to 0.1.
         dropout_att (float): the dropout rate used for the attention layer
@@ -97,12 +118,12 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         input_example_length = torch.randint(1, max_dim, (max_batch,)).to(dev)
 
         if hasattr(self, 'export_cache_support') and self.export_cache_support:
-            cache_last_channel = torch.randn(self.n_layers, max_batch, self.conv_context_size[0], self.d_model).to(dev)
-            cache_last_time = torch.randn(self.n_layers, max_batch, self.d_model, max_dim).to(dev)
+            cache_last_channel = torch.randn(self.n_layers, max_batch, max_dim, self.d_model).to(dev)
+            cache_last_time = torch.randn(self.n_layers, max_batch, self.d_model, self.conv_context_size[0]).to(dev)
+            all_input_example = tuple([input_example, input_example_length, cache_last_channel, cache_last_time])
         else:
-            cache_last_channel = cache_last_time = None
+            all_input_example = tuple([input_example, input_example_length])
 
-        all_input_example = tuple([input_example, input_example_length, cache_last_channel, cache_last_time])
         return all_input_example
 
     @property
@@ -153,6 +174,9 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         subsampling='striding',
         subsampling_factor=4,
         subsampling_conv_channels=-1,
+        reduction=None,
+        reduction_position=None,
+        reduction_factor=1,
         ff_expansion_factor=4,
         self_attention_model='rel_pos',
         n_heads=4,
@@ -165,6 +189,7 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         conv_norm_type='batch_norm',
         conv_context_size=None,
         dropout=0.1,
+        dropout_pre_encoder=0.1,
         dropout_emb=0.1,
         dropout_att=0.0,
     ):
@@ -176,6 +201,7 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         self.scale = math.sqrt(self.d_model)
         self.att_context_style = att_context_style
         self.subsampling_factor = subsampling_factor
+        self.self_attention_model = self_attention_model
 
         if att_context_size:
             self.att_context_size = list(att_context_size)
@@ -228,6 +254,7 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         else:
             self.xscale = None
 
+        # Subsampling
         if subsampling_conv_channels == -1:
             subsampling_conv_channels = d_model
         if subsampling and subsampling_factor > 1:
@@ -246,12 +273,23 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
                     feat_in=feat_in,
                     feat_out=d_model,
                     conv_channels=subsampling_conv_channels,
-                    activation=nn.ReLU(),
+                    activation=nn.ReLU(True),
                     is_causal=causal_downsampling,
                 )
         else:
             self.pre_encode = nn.Linear(feat_in, d_model)
 
+        # Reduction
+        if reduction and reduction_factor > 1:
+            assert reduction_position >= -1 and reduction_position < n_layers
+            self.reduction_subsampling = SubsamplingReductionModule(
+                reduction=reduction, d_model=d_model, reduction_factor=reduction_factor,
+            )
+            self.reduction_position = reduction_position
+        else:
+            self.reduction_subsampling = None
+            self.reduction_position = None
+
         self._feat_out = d_model
 
         if not untie_biases and self_attention_model == "rel_pos":
@@ -265,8 +303,20 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
             pos_bias_v = None
 
         self.pos_emb_max_len = pos_emb_max_len
+        self.att_mask = None
         if self_attention_model == "rel_pos":
             self.pos_enc = RelPositionalEncoding(
+                d_model=d_model,
+                dropout_rate=dropout_pre_encoder,
+                max_len=pos_emb_max_len,
+                xscale=self.xscale,
+                dropout_rate_emb=dropout_emb,
+            )
+        elif self_attention_model == 'rel_pos_local_attn':
+            if max(att_context_size) <= 0:
+                raise ValueError("When using local attention, context size must be set > 0")
+            self.pos_enc = LocalAttRelPositionalEncoding(
+                att_context_size=att_context_size,
                 d_model=d_model,
                 dropout_rate=dropout,
                 max_len=pos_emb_max_len,
@@ -277,7 +327,7 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
             pos_bias_u = None
             pos_bias_v = None
             self.pos_enc = PositionalEncoding(
-                d_model=d_model, dropout_rate=dropout, max_len=pos_emb_max_len, xscale=self.xscale
+                d_model=d_model, dropout_rate=dropout_pre_encoder, max_len=pos_emb_max_len, xscale=self.xscale
             )
         else:
             raise ValueError(f"Not valid self_attention_model: '{self_attention_model}'!")
@@ -334,25 +384,28 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         device = next(self.parameters()).device
         self.pos_enc.extend_pe(max_audio_length, device)
 
-        att_mask = torch.ones(1, max_audio_length, max_audio_length, dtype=torch.bool, device=device)
-        if self.chunk_size is None:
-            if self.att_context_size[0] >= 0:
-                att_mask = att_mask.triu(diagonal=-self.att_context_size[0])
-            if self.att_context_size[1] >= 0:
-                att_mask = att_mask.tril(diagonal=self.att_context_size[1])
-        else:
-            chunk_idx = torch.arange(0, max_audio_length, dtype=torch.int, device=att_mask.device)
-            chunk_idx = torch.div(chunk_idx, self.chunk_size, rounding_mode="trunc")
-            diff_chunks = chunk_idx.unsqueeze(1) - chunk_idx.unsqueeze(0)
-            chunked_limited_mask = torch.logical_and(
-                torch.le(diff_chunks, self.left_chunks_num), torch.ge(diff_chunks, 0)
-            )
-            att_mask = torch.logical_and(att_mask, chunked_limited_mask.unsqueeze(0))
+        if self.self_attention_model != "rel_pos_local_attn":
+            att_mask = torch.ones(1, max_audio_length, max_audio_length, dtype=torch.bool, device=device)
+            if self.chunk_size is None:
+                if self.att_context_size[0] >= 0:
+                    att_mask = att_mask.triu(diagonal=-self.att_context_size[0])
+                if self.att_context_size[1] >= 0:
+                    att_mask = att_mask.tril(diagonal=self.att_context_size[1])
+            else:
+                chunk_idx = torch.arange(0, max_audio_length, dtype=torch.int, device=att_mask.device)
+                chunk_idx = torch.div(chunk_idx, self.chunk_size, rounding_mode="trunc")
+                diff_chunks = chunk_idx.unsqueeze(1) - chunk_idx.unsqueeze(0)
+                chunked_limited_mask = torch.logical_and(
+                    torch.le(diff_chunks, self.left_chunks_num), torch.ge(diff_chunks, 0)
+                )
+                att_mask = torch.logical_and(att_mask, chunked_limited_mask.unsqueeze(0))
 
-        if hasattr(self, 'att_mask'):
-            self.att_mask = att_mask
+            if hasattr(self, 'att_mask'):
+                self.att_mask = att_mask
+            else:
+                self.register_buffer('att_mask', att_mask, persistent=False)
         else:
-            self.register_buffer('att_mask', att_mask, persistent=False)
+            self.att_mask = None
 
     @typecheck()
     def forward(self, audio_signal, length, cache_last_channel=None, cache_last_time=None):
@@ -361,10 +414,10 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
 
         if length is None:
             length = audio_signal.new_full(
-                audio_signal.size(0), max_audio_length, dtype=torch.int32, device=audio_signal.device
+                (audio_signal.size(0),), max_audio_length, dtype=torch.int32, device=audio_signal.device
             )
 
-        if cache_last_channel is not None:
+        if cache_last_time is not None:
             cache_last_time_next = torch.zeros_like(cache_last_time)
         else:
             cache_last_time_next = None
@@ -383,7 +436,9 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
 
         max_audio_length = audio_signal.size(1)
 
-        # Create the self-attention and padding masks
+        if self.reduction_position is not None and cache_last_channel is not None:
+            raise ValueError("Caching with reduction feature is not supported yet!")
+
         if cache_last_channel is not None:
             last_channel_num, bs, cache_len, channel_size = cache_last_channel.size()
             cache_keep_size = max_audio_length - self.streaming_cfg.cache_drop_size
@@ -394,31 +449,23 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
             )
             max_audio_length += cache_len
             padding_length = length + cache_len
-            audio_signal, pos_emb = self.pos_enc(x=audio_signal, cache_len=cache_len)
         else:
             padding_length = length
             cache_last_channel_next = None
+            cache_len = 0
+
+        if self.self_attention_model == 'abs_pos':
             audio_signal, pos_emb = self.pos_enc(x=audio_signal)
+        else:
+            audio_signal, pos_emb = self.pos_enc(x=audio_signal, cache_len=cache_len)
 
-        # pad_mask is the masking to be used to ignore paddings
-        pad_mask = torch.arange(0, max_audio_length, device=audio_signal.device).expand(
-            padding_length.size(0), -1
-        ) < padding_length.unsqueeze(-1)
-
-        # pad_mask_for_att_mask is the mask which helps to ignore paddings
-        pad_mask_for_att_mask = pad_mask.unsqueeze(1).repeat([1, max_audio_length, 1])
-        pad_mask_for_att_mask = torch.logical_and(pad_mask_for_att_mask, pad_mask_for_att_mask.transpose(1, 2))
-        # att_mask is the masking to be used by the MHA layers to ignore the tokens not supposed to be visible
-        att_mask = self.att_mask[:, :max_audio_length, :max_audio_length]
-        # paddings should also get ignored, so pad_mask_for_att_mask is used to ignore their corresponding scores
-        att_mask = torch.logical_and(pad_mask_for_att_mask, att_mask.to(pad_mask_for_att_mask.device))
+        # Create the self-attention and padding masks
+        pad_mask, att_mask = self._create_masks(max_audio_length, padding_length, audio_signal.device)
 
         if cache_last_channel is not None:
             pad_mask = pad_mask[:, cache_len:]
-            att_mask = att_mask[:, cache_len:]
-
-        pad_mask = ~pad_mask
-        att_mask = ~att_mask
+            if self.att_mask is not None:
+                att_mask = att_mask[:, cache_len:]
 
         for lth, layer in enumerate(self.layers):
             audio_signal = layer(
@@ -426,15 +473,27 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
                 att_mask=att_mask,
                 pos_emb=pos_emb,
                 pad_mask=pad_mask,
-                cache_last_time=cache_last_time,
                 cache_last_channel=cache_last_channel,
-                cache_last_time_next=cache_last_time_next,
+                cache_last_time=cache_last_time,
                 cache_last_channel_next=cache_last_channel_next,
+                cache_last_time_next=cache_last_time_next,
             )
 
+            if self.reduction_position == lth:
+                audio_signal, length = self.reduction_subsampling(x=audio_signal, lengths=length)
+                max_audio_length = audio_signal.size(1)
+                # Don't update the audio_signal here because then it will again scale the audio_signal
+                # and cause an increase in the WER
+                _, pos_emb = self.pos_enc(x=audio_signal, cache_len=cache_len)
+                pad_mask, att_mask = self._create_masks(max_audio_length, length, audio_signal.device)
+
         if self.out_proj is not None:
             audio_signal = self.out_proj(audio_signal)
 
+        # Reduction
+        if self.reduction_position == -1:
+            audio_signal, length = self.reduction_subsampling(x=audio_signal, lengths=length)
+
         audio_signal = torch.transpose(audio_signal, 1, 2)
 
         if cache_last_channel is not None:
@@ -442,6 +501,29 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         else:
             return audio_signal, length
 
+    def _create_masks(self, max_audio_length, padding_length, device):
+        # pad_mask is the masking to be used to ignore paddings
+        pad_mask = torch.arange(0, max_audio_length, device=device).expand(
+            padding_length.size(0), -1
+        ) < padding_length.unsqueeze(-1)
+
+        if self.att_mask is not None:
+            # pad_mask_for_att_mask is the mask which helps to ignore paddings
+            pad_mask_for_att_mask = pad_mask.unsqueeze(1).repeat([1, max_audio_length, 1])
+            pad_mask_for_att_mask = torch.logical_and(pad_mask_for_att_mask, pad_mask_for_att_mask.transpose(1, 2))
+            # att_mask is the masking to be used by the MHA layers to ignore the tokens not supposed to be visible
+            att_mask = self.att_mask[:, :max_audio_length, :max_audio_length]
+            # paddings should also get ignored, so pad_mask_for_att_mask is used to ignore their corresponding scores
+            att_mask = torch.logical_and(pad_mask_for_att_mask, att_mask.to(pad_mask_for_att_mask.device))
+
+            att_mask = ~att_mask
+        else:
+            att_mask = None
+
+        pad_mask = ~pad_mask
+
+        return pad_mask, att_mask
+
     def enable_pad_mask(self, on=True):
         # On inference, user may choose to disable pad mask
         mask = self.use_pad_mask
@@ -463,7 +545,7 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
         """
         streaming_cfg = CacheAwareStreamingConfig()
         if chunk_size is not None:
-            if chunk_size <= 1:
+            if chunk_size < 1:
                 raise ValueError("chunk_size needs to be a number larger or equal to one.")
             lookahead_steps = chunk_size - 1
             streaming_cfg.cache_drop_size = chunk_size - shift_size
@@ -561,6 +643,124 @@ class ConformerEncoder(NeuralModule, StreamingEncoder, Exportable):
 
         return cache_last_channel, cache_last_time
 
+    def change_attention_model(
+        self,
+        self_attention_model: str = None,
+        att_context_size: List[int] = None,
+        update_config: bool = True,
+        device: torch.device = None,
+    ):
+
+        """
+        Update the self_attention_model which changes the positional encoding and attention layers.
+    
+        Args:
+            self_attention_model (str): type of the attention layer and positional encoding
+                'rel_pos': relative positional embedding and Transformer-XL
+                'rel_pos_local_attn': relative positional embedding and Transformer-XL with local attention using
+                    overlapping windows. Attention context is determined by att_context_size parameter.
+                'abs_pos': absolute positional embedding and Transformer
+                If None is provided, the self_attention_model isn't changed. Defauts to None.
+            att_context_size (List[int]): List of 2 ints corresponding to left and right attention context sizes,
+                or None to keep as it is. Defauts to None.
+            update_config (bool): Whether to update the config or not with the new attention model.
+                Defaults to True.
+            device (torch.device): If provided, new layers will be moved to the device.
+                Defaults to None.
+        """
+
+        if att_context_size:
+            att_context_size = list(att_context_size)
+        else:
+            att_context_size = self._cfg.att_context_size
+
+        if self_attention_model is None:
+            self_attention_model = self._cfg.self_attention_model
+
+        if self_attention_model == 'rel_pos_local_attn' and max(att_context_size) <= 0:
+            raise ValueError("When using local attention, context size must be set > 0")
+
+        if self_attention_model == "rel_pos":
+            self.att_mask = None
+            new_pos_enc = RelPositionalEncoding(
+                d_model=self._cfg.d_model,
+                dropout_rate=self._cfg.dropout,
+                max_len=self._cfg.pos_emb_max_len,
+                xscale=self.xscale,
+                dropout_rate_emb=self._cfg.dropout_emb,
+            )
+        elif self_attention_model == 'rel_pos_local_attn':
+            new_pos_enc = LocalAttRelPositionalEncoding(
+                att_context_size=att_context_size,
+                d_model=self._cfg.d_model,
+                dropout_rate=self._cfg.dropout,
+                max_len=self._cfg.pos_emb_max_len,
+                xscale=self.xscale,
+                dropout_rate_emb=self._cfg.dropout_emb,
+            )
+        elif self_attention_model == "abs_pos":
+            new_pos_enc = PositionalEncoding(
+                d_model=self._cfg.d_model,
+                dropout_rate=self._cfg.dropout,
+                max_len=self._cfg.pos_emb_max_len,
+                xscale=self.xscale,
+            )
+        else:
+            raise ValueError(f"Not valid self_attention_model: '{self_attention_model}'!")
+
+        if device is not None:
+            new_pos_enc = new_pos_enc.to(device=device)
+        del self.pos_enc
+        self.pos_enc = new_pos_enc
+        self.self_attention_model = self_attention_model
+        self.att_context_size = att_context_size
+        self.set_max_audio_length(self.pos_emb_max_len)
+
+        for name, m in self.named_modules():
+            if type(m) == ConformerLayer:
+
+                if self_attention_model == 'rel_pos':
+                    new_attn = RelPositionMultiHeadAttention(
+                        n_head=self._cfg.n_heads,
+                        n_feat=self._cfg.d_model,
+                        dropout_rate=self._cfg.dropout_att,
+                        max_cache_len=att_context_size[0],
+                        pos_bias_u=None,
+                        pos_bias_v=None,
+                    )
+                elif self_attention_model == 'rel_pos_local_attn':
+                    new_attn = RelPositionMultiHeadAttentionLongformer(
+                        n_head=self._cfg.n_heads,
+                        n_feat=self._cfg.d_model,
+                        dropout_rate=self._cfg.dropout_att,
+                        max_cache_len=att_context_size[0],
+                        att_context_size=att_context_size,
+                        pos_bias_u=None,
+                        pos_bias_v=None,
+                    )
+                elif self_attention_model == 'abs_pos':
+                    new_attn = MultiHeadAttention(
+                        n_head=self._cfg.n_heads,
+                        n_feat=self._cfg.d_model,
+                        dropout_rate=self._cfg.dropout_att,
+                        max_cache_len=att_context_size[0],
+                    )
+                else:
+                    raise ValueError(
+                        f"'{self_attention_model}' is not not a valid value for 'self_attention_model', "
+                        f"valid values can be from ['rel_pos', 'rel_pos_local_attn', 'abs_pos']"
+                    )
+                if device is not None:
+                    new_attn = new_attn.to(device=device)
+                new_attn.load_state_dict(m.self_attn.state_dict(), strict=False)
+                del m.self_attn
+                m.self_attn = new_attn
+                m.self_attention_model = self_attention_model
+
+        if update_config:
+            self._cfg.self_attention_model = self_attention_model
+            self._cfg.att_context_size = att_context_size
+
 
 class ConformerEncoderAdapter(ConformerEncoder, adapter_mixins.AdapterModuleMixin):
 
@@ -589,9 +789,40 @@ class ConformerEncoderAdapter(ConformerEncoder, adapter_mixins.AdapterModuleMixi
         cfg = adapter_utils.update_adapter_cfg_input_dim(self, cfg, module_dim=self.d_model)
         return cfg
 
+    def get_accepted_adapter_types(self,) -> Set[type]:
+        types = super().get_accepted_adapter_types()
+
+        if len(types) == 0:
+            self.set_accepted_adapter_types(
+                [
+                    adapter_utils.LINEAR_ADAPTER_CLASSPATH,
+                    adapter_utils.MHA_ADAPTER_CLASSPATH,
+                    adapter_utils.RELMHA_ADAPTER_CLASSPATH,
+                ]
+            )
+            types = self.get_accepted_adapter_types()
+        return types
+
 
 """
 Register any additional information
 """
 if adapter_mixins.get_registered_adapter(ConformerEncoder) is None:
     adapter_mixins.register_adapter(base_class=ConformerEncoder, adapter_class=ConformerEncoderAdapter)
+
+
+@dataclass
+class ConformerChangeConfig:
+    # Change self_attention_model for Conformer
+    # Options:
+    #  'rel_pos': relative positional embedding and Transformer-XL
+    #  'rel_pos_local_attn': relative positional embedding and Transformer-XL with local attention using
+    #   overlapping chunks. Attention context is determined by att_context_size parameter.
+    #  'abs_pos': absolute positional embedding and Transformer
+    # If None is provided, self_attention_model is not changed.
+    self_attention_model: Optional[str] = None
+
+    # Change the attention context size by providing 2 integers,
+    # corresponding to left and right context, or -1 for full context.
+    # If None is provided, the attention context size isn't changed.
+    att_context_size: Optional[List[int]] = None
