@@ -13,7 +13,7 @@
 # limitations under the License.
 from collections import OrderedDict
 from dataclasses import dataclass, field
-from typing import List, Optional, Union
+from typing import List, Optional, Set, Union
 
 import torch
 import torch.distributed
@@ -441,6 +441,9 @@ class ConvASRDecoder(NeuralModule, Exportable, adapter_mixins.AdapterModuleMixin
         )
         self.apply(lambda x: init_weights(x, mode=init_mode))
 
+        accepted_adapters = [adapter_utils.LINEAR_ADAPTER_CLASSPATH]
+        self.set_accepted_adapter_types(accepted_adapters)
+
     @typecheck()
     def forward(self, encoder_output):
         # Adapter module forward step
@@ -885,6 +888,8 @@ class ConvASREncoderAdapter(ConvASREncoder, adapter_mixins.AdapterModuleMixin):
     def add_adapter(self, name: str, cfg: dict):
         for jasper_block in self.encoder:  # type: adapter_mixins.AdapterModuleMixin
             cfg = self._update_adapter_cfg_input_dim(jasper_block, cfg)
+
+            jasper_block.set_accepted_adapter_types(self.get_accepted_adapter_types())
             jasper_block.add_adapter(name, cfg)
 
     def is_adapter_available(self) -> bool:
@@ -906,6 +911,16 @@ class ConvASREncoderAdapter(ConvASREncoder, adapter_mixins.AdapterModuleMixin):
         cfg = adapter_utils.update_adapter_cfg_input_dim(self, cfg, module_dim=block.planes)
         return cfg
 
+    def get_accepted_adapter_types(self,) -> Set[type]:
+        types = super().get_accepted_adapter_types()
+
+        if len(types) == 0:
+            self.set_accepted_adapter_types(
+                [adapter_utils.LINEAR_ADAPTER_CLASSPATH,]
+            )
+            types = self.get_accepted_adapter_types()
+        return types
+
 
 @dataclass
 class JasperEncoderConfig:
@@ -934,7 +949,7 @@ class JasperEncoderConfig:
 @dataclass
 class ConvASREncoderConfig:
     _target_: str = 'nemo.collections.asr.modules.ConvASREncoder'
-    jasper: Optional[JasperEncoderConfig] = field(default_factory=list)
+    jasper: Optional[List[JasperEncoderConfig]] = field(default_factory=list)
     activation: str = MISSING
     feat_in: int = MISSING
     normalization_mode: str = "batch"
