@@ -17,5 +17,5 @@ from nemo.collections.common.losses.bce_logits_loss import BCEWithLogitsLoss
 from nemo.collections.common.losses.cross_entropy import CrossEntropyLoss, NLLLoss
 from nemo.collections.common.losses.mse_loss import MSELoss
 from nemo.collections.common.losses.multi_similarity_loss import MultiSimilarityLoss
-from nemo.collections.common.losses.smoothed_cross_entropy import SmoothedCrossEntropyLoss
+from nemo.collections.common.losses.smoothed_cross_entropy import SmoothedCrossEntropyLoss, SmoothedNLLLoss
 from nemo.collections.common.losses.spanning_loss import SpanningLoss
