@@ -26,14 +26,15 @@ __all__ = ['SentencePieceTokenizer', 'create_spt_model']
 
 
 class SentencePieceTokenizer(TokenizerSpec):
-    '''
+    """
     Sentencepiecetokenizer https://github.com/google/sentencepiece.
+    
         Args:
         model_path: path to sentence piece tokenizer model. To create the model use create_spt_model()
         special_tokens: either list of special tokens or dictionary of token name to token value
-        legacy: when set to True, the previous behavior of the SentecePiece wrapper will be restored, 
+        legacy: when set to True, the previous behavior of the SentecePiece wrapper will be restored,
             including the possibility to add special tokens inside wrapper.
-    '''
+    """
 
     def __init__(
         self, model_path: str, special_tokens: Optional[Union[Dict[str, str], List[str]]] = None, legacy: bool = False
@@ -272,6 +273,7 @@ def create_spt_model(
     byte_fallback: bool = False,
     split_digits: bool = False,
     split_by_whitespace: bool = True,
+    split_by_unicode_script: bool = True,
 ):
     """
     Creates sentence piece tokenizer model from data file.
@@ -297,6 +299,7 @@ def create_spt_model(
         byte_fallback: If <unk>, fallback to a byte sequence of the character.
         split_digits: If true, digits are split into individual tokens.
         split_by_whitespace: Whether to respect white space while creating subwords. If False, will learn merges across whitespace.
+        split_by_unicode_script: Whether to include multiple Unicode scripts. Ex. is Arabic diacritics which are considered part of the letter (عِدَّةُ)
     """
 
     if not data_file or not os.path.exists(data_file):
@@ -363,6 +366,9 @@ def create_spt_model(
     if not split_by_whitespace:
         cmd += " --split_by_whitespace=false"
 
+    if not split_by_unicode_script:
+        cmd += " --split_by_unicode_script=false"
+
     sentencepiece.SentencePieceTrainer.Train(cmd)
 
     # Add BERT control symbols
