@@ -13,32 +13,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import abc
-import ctypes.util
 import itertools
 import string
+from abc import ABC, abstractmethod
 from contextlib import contextmanager
-from logging import ERROR, getLogger
 from typing import List, Optional
 
 from nemo_text_processing.g2p.data.data_utils import (
+    any_locale_text_preprocessing,
+    chinese_text_preprocessing,
     english_text_preprocessing,
-    german_text_preprocessing,
-    ipa_word_tokenize,
+    spanish_text_preprocessing,
 )
-from nemo_text_processing.g2p.modules import IPAG2P
-from phonemizer.backend import EspeakBackend
 
 from nemo.collections.common.tokenizers.text_to_speech.ipa_lexicon import (
     get_grapheme_character_set,
-    get_ipa_character_set,
     get_ipa_punctuation_list,
+    validate_locale,
 )
 from nemo.utils import logging
 from nemo.utils.decorators import experimental
 
 
-class BaseTokenizer(abc.ABC):
+class BaseTokenizer(ABC):
     PAD, BLANK, OOV = '<pad>', '<blank>', '<oov>'
 
     def __init__(self, tokens, *, pad=PAD, blank=BLANK, oov=OOV, sep='', add_blank_at=None):
@@ -55,6 +52,9 @@ class BaseTokenizer(abc.ABC):
         super().__init__()
 
         tokens = list(tokens)
+        # TODO @xueyang: in general, IDs of pad, sil, blank, and oov are preserved ahead instead of dynamically
+        #  assigned according to the number of tokens. The downside of using dynamical assignment leads to different IDs
+        #  for each.
         self.pad, tokens = len(tokens), tokens + [pad]  # Padding
 
         if add_blank_at is not None:
@@ -79,7 +79,7 @@ class BaseTokenizer(abc.ABC):
     def __call__(self, text: str) -> List[int]:
         return self.encode(text)
 
-    @abc.abstractmethod
+    @abstractmethod
     def encode(self, text: str) -> List[int]:
         """Turns str text into int tokens."""
         pass
@@ -91,6 +91,7 @@ class BaseTokenizer(abc.ABC):
 
 class BaseCharsTokenizer(BaseTokenizer):
     # fmt: off
+    # TODO @xueyang: unify definition of the default PUNCT_LIST and import from ipa_lexicon.py
     PUNCT_LIST = (  # Derived from LJSpeech and "/" additionally
         ',', '.', '!', '?', '-',
         ':', ';', '/', '"', '(',
@@ -144,13 +145,13 @@ class BaseCharsTokenizer(BaseTokenizer):
 
         text = self.text_preprocessing_func(text)
         for c in text:
-            # Add space if last one isn't one
+            # Add a whitespace if the current char is a whitespace while the previous char is not a whitespace.
             if c == space and len(cs) > 0 and cs[-1] != space:
                 cs.append(c)
-            # Add next char
+            # Add the current char that is an alphanumeric or an apostrophe.
             elif (c.isalnum() or c == "'") and c in tokens:
                 cs.append(c)
-            # Add punct
+            # Add a punctuation that has a single char.
             elif (c in self.PUNCT_LIST) and self.punct:
                 cs.append(c)
             # Warn about unknown char
@@ -201,25 +202,22 @@ class EnglishCharsTokenizer(BaseCharsTokenizer):
 
 
 class GermanCharsTokenizer(BaseCharsTokenizer):
-    # fmt: off
-    PUNCT_LIST = (  # Derived from LJSpeech and "/" additionally
-        ',', '.', '!', '?', '-',
-        ':', ';', '/', '"', '(',
-        ')', '[', ']', '{', '}',
-    )
-    # fmt: on
+
+    _LOCALE = "de-DE"
+    _PUNCT_LIST = get_ipa_punctuation_list(_LOCALE)
+    _CHARSET_STR = get_grapheme_character_set(locale=_LOCALE, case="mixed")
 
     def __init__(
         self,
+        chars=_CHARSET_STR,
         punct=True,
         apostrophe=True,
         add_blank_at=None,
         pad_with_space=False,
-        non_default_punct_list=None,
-        text_preprocessing_func=german_text_preprocessing,
-        phonemes=True,
+        non_default_punct_list=_PUNCT_LIST,
+        text_preprocessing_func=any_locale_text_preprocessing,
     ):
-        """Deutsch char-based tokenizer.
+        """German grapheme-based tokenizer.
         Args:
             punct: Whether to reserve grapheme for basic punctuation or not.
             apostrophe: Whether to use apostrophe or not.
@@ -227,16 +225,11 @@ class GermanCharsTokenizer(BaseCharsTokenizer):
              if None then no blank in labels.
             pad_with_space: Whether to pad text with spaces at the beginning and at the end or not.
             non_default_punct_list: List of punctuation marks which will be used instead default.
-            text_preprocessing_func: Text preprocessing function for correct execution of the tokenizer.
-             Currently, it only applies lower() function.
+            text_preprocessing_func: Text preprocessing function for correct execution of the tokenizer. By default, it
+            would keep any word unchanged.
         """
-
-        de_alphabet = "abcdefghijklmnopqrstuvwxyzäöüß"
-        if phonemes:
-            de_ipa = "ʊʃŋɜːɛɾəɪçɔøɡœɑÜ„1Q̃ɒʒÄɹÖʌθàó̈ðéɐá"
-            de_alphabet += de_ipa
         super().__init__(
-            chars=de_alphabet,
+            chars=chars,
             punct=punct,
             apostrophe=apostrophe,
             add_blank_at=add_blank_at,
@@ -246,6 +239,35 @@ class GermanCharsTokenizer(BaseCharsTokenizer):
         )
 
 
+class SpanishCharsTokenizer(BaseCharsTokenizer):
+
+    PUNCT_LIST = get_ipa_punctuation_list("es-ES")
+
+    def __init__(
+        self, punct=True, apostrophe=True, add_blank_at=None, pad_with_space=False, non_default_punct_list=None,
+    ):
+        """Spanish grapheme tokenizer.
+        Args:
+            punct: Whether to reserve grapheme for basic punctuation or not.
+            apostrophe: Whether to use apostrophe or not.
+            add_blank_at: Add blank to labels in the specified order ("last") or after tokens (any non None),
+             if None then no blank in labels.
+            pad_with_space: Whether to pad text with spaces at the beginning and at the end or not.
+            non_default_punct_list: List of punctuation marks which will be used instead default.
+        """
+
+        es_alphabet = "abcdefghijklmnopqrstuvwxyzáéíñóúü"
+        super().__init__(
+            chars=es_alphabet,
+            punct=punct,
+            apostrophe=apostrophe,
+            add_blank_at=add_blank_at,
+            pad_with_space=pad_with_space,
+            non_default_punct_list=non_default_punct_list,
+            text_preprocessing_func=spanish_text_preprocessing,
+        )
+
+
 class GermanPhonemesTokenizer(BaseCharsTokenizer):
     # fmt: off
     PUNCT_LIST = (  # Derived from LJSpeech and "/" additionally
@@ -262,7 +284,7 @@ class GermanPhonemesTokenizer(BaseCharsTokenizer):
         add_blank_at=None,
         pad_with_space=False,
         non_default_punct_list=None,
-        text_preprocessing_func=german_text_preprocessing,
+        text_preprocessing_func=any_locale_text_preprocessing,
     ):
         """Deutsch phoneme-based tokenizer.
         Args:
@@ -371,7 +393,7 @@ class EnglishPhonemesTokenizer(BaseTokenizer):
             pad_with_space: Whether to pad text with spaces at the beginning and at the end or not.
             text_preprocessing_func: Text preprocessing function for correct execution of the tokenizer.
              Basically, it replaces all non-unicode characters with unicode ones.
-             Note that lower() function shouldn't applied here, in case the text contains phonemes (it will be handled by g2p).
+             Note that lower() function shouldn't be applied here, in case the text contains phonemes (it will be handled by g2p).
         """
 
         self.phoneme_probability = None
@@ -478,17 +500,10 @@ class EnglishPhonemesTokenizer(BaseTokenizer):
 
 @experimental
 class IPATokenizer(BaseTokenizer):
-    # fmt: off
-    PUNCT_LIST = (  # Derived from LJSpeech and "/" additionally
-        ',', '.', '!', '?', '-',
-        ':', ';', '/', '"', '(',
-        ')', '[', ']', '{', '}',
-    )
-    # fmt: on
-
     def __init__(
         self,
         g2p,
+        locale="en-US",
         punct=True,
         non_default_punct_list=None,
         *,
@@ -499,11 +514,13 @@ class IPATokenizer(BaseTokenizer):
         sep='|',  # To be able to distinguish between symbols
         add_blank_at=None,
         pad_with_space=False,
-        text_preprocessing_func=lambda text: english_text_preprocessing(text, lower=False),
     ):
         """General-purpose IPA-based tokenizer.
         Args:
             g2p: Grapheme to phoneme module, should be IPAG2P or some subclass thereof.
+            locale: Locale used to determine default text processing logic and punctuation.
+                Supports ["en-US", "de-DE", "es-ES"]. Defaults to "en-US".
+                Specify None if implementing custom logic for a new locale.
             punct: Whether to reserve grapheme for basic punctuation or not.
             non_default_punct_list: List of punctuation marks which will be used instead default, if any.
             space: Space token as string.
@@ -512,12 +529,8 @@ class IPATokenizer(BaseTokenizer):
             oov: OOV token as string.
             sep: Separation token as string.
             add_blank_at: Add blank to labels in the specified order ("last") or after tokens (any non None),
-             if None then no blank in labels.
+                if None then no blank in labels.
             pad_with_space: Whether to pad text with spaces at the beginning and at the end or not.
-            text_preprocessing_func: Text preprocessing function for correct execution of the tokenizer.
-             Basically, it replaces all non-unicode characters with unicode ones.
-             Note that lower() function shouldn't applied here, in case the text contains phonemes (it will be handled by g2p).
-             Defaults to English text preprocessing.
         """
         if not hasattr(g2p, "symbols"):
             logging.error(
@@ -527,6 +540,9 @@ class IPATokenizer(BaseTokenizer):
             )
             raise ValueError("G2P modules passed into the IPATokenizer must have `symbols` defined.")
 
+        if locale is not None:
+            validate_locale(locale)
+
         self.phoneme_probability = None
         if hasattr(g2p, "phoneme_probability"):
             self.phoneme_probability = g2p.phoneme_probability
@@ -539,8 +555,11 @@ class IPATokenizer(BaseTokenizer):
 
         if punct:
             if non_default_punct_list is not None:
-                self.PUNCT_LIST = non_default_punct_list
-            tokens.update(self.PUNCT_LIST)
+                self.punct_list = non_default_punct_list
+            else:
+                self.punct_list = get_ipa_punctuation_list(locale)
+
+            tokens.update(self.punct_list)
 
         # Sort to ensure that vocab is in the same order every time
         tokens = sorted(list(tokens))
@@ -558,25 +577,39 @@ class IPATokenizer(BaseTokenizer):
         self.punct = punct
         self.pad_with_space = pad_with_space
 
-        self.text_preprocessing_func = text_preprocessing_func
         self.g2p = g2p
 
-    def encode(self, text):
-        """See base class for more information."""
+        if locale == "en-US":
+            self.text_preprocessing_func = lambda text: english_text_preprocessing(text, lower=False)
+        else:
+            self.text_preprocessing_func = any_locale_text_preprocessing
 
+    def encode(self, text: str) -> List[int]:
+        """See base class for more information."""
+        # normalize the input text with "NFC" form.
         text = self.text_preprocessing_func(text)
-        g2p_text = self.g2p(text)  # Double-check this
+
+        # transliterate the text into phoneme sequences and/or grapheme sequences.
+        g2p_text = self.g2p(text)
+
         return self.encode_from_g2p(g2p_text, text)
 
-    def encode_from_g2p(self, g2p_text: List[str], raw_text: Optional[str] = None):
+    def encode_from_g2p(self, g2p_text: List[str], raw_text: Optional[str] = None) -> List[int]:
         """
-        Encodes text that has already been run through G2P.
-        Called for encoding to tokens after text preprocessing and G2P.
+        Tokenize the `g2p_text` that has been already run through G2P. Each item in the `g2p_text` would be encoded as
+        one of the integer IDs predefined in `self._token2id`. Note that this function should be called after
+        `self.text_preprocessing_func` and `self.g2p` functions
 
         Args:
-            g2p_text: G2P's output, could be a mixture of phonemes and graphemes,
-                e.g. "see OOV" -> ['ˈ', 's', 'i', ' ', 'O', 'O', 'V']
-            raw_text: original raw input
+            g2p_text (List[str]): a sequence of tokens from G2P's output. It could be a sequence of phonemes, a sequence
+                of graphemes, or a mixture of both. For example, `['ˈ', 's', 'i', ' ', '#O', '#O', '#V']`, which is the
+                G2P's output of the text "see OOV", where '#' is prepended to each grapheme in order to distinguish
+                graphemes from phonemes if there are overlaps in between. The prefix '#' can be customized in
+                `nemo_text_processing.g2p.modules.IPAG2P.grapheme_prefix`.
+            raw_text (str): the original text after calling `self.text_preprocessing_func`. It is optional. It is only
+                used to deliver a warning message that some graphemes from the original text are skipped.
+
+        Returns: a list of integer IDs that tokenize the `g2p_text`.
         """
         ps, space, tokens = [], self.tokens[self.space], set(self.tokens)
         for p in g2p_text:
@@ -586,7 +619,7 @@ class IPATokenizer(BaseTokenizer):
             elif p in tokens:
                 # Add next phoneme or char (if chars=True)
                 ps.append(p)
-            elif (p in self.PUNCT_LIST) and self.punct:
+            elif (p in self.punct_list) and self.punct:
                 # Add punct
                 ps.append(p)
             elif p != space:
@@ -617,134 +650,119 @@ class IPATokenizer(BaseTokenizer):
                 self.g2p.phoneme_probability = self.phoneme_probability
 
 
-@experimental
-class PhonemizerTokenizer(IPATokenizer):
+class ChinesePhonemesTokenizer(BaseTokenizer):
     # fmt: off
-    ESPEAK_SYMBOLS = ('ˈ', 'ˌ', "'", '-', 'ː')
-    # fmt: on
+    PRONUNCIATION_LIST = ['#' + i for i in ['^', 'A', 'AI', 'AN', 'ANG', 'AO', 'B', 'C', 'CH', 'D', 
+                    'E', 'EI', 'EN', 'ENG', 'ER', 'F', 'G', 'H', 'I', 'IE', 
+                    'IN', 'ING', 'IU', 'J', 'K', 'L', 'M', 'N', 'O', 'ONG', 
+                    'OU', 'P', 'Q', 'R', 'S', 'SH', 'T', 'U', 'UI', 'UN', 
+                    'V', 'VE', 'VN', 'W', 'X', 'Y', 'Z', 'ZH']]
+    TONES_LIST = ['#1', '#2', '#3', '#4', '#5']
+    PUNCT_LIST = (  # Derived from LJSpeech and "/" additionally
+        ',', '.', '!', '?', '-',
+        ':', ';', '/', '"', '(',
+        ')', '[', ']', '{', '}',
+    )
+    ZH_PUNCT_LIST = list("，。？！；：、‘’“”（）【】「」《》") + list(PUNCT_LIST)
 
     def __init__(
         self,
-        language,
-        phoneme_dict,
+        g2p,
         punct=True,
-        pad_with_space=False,
-        non_default_chars=None,
         non_default_punct_list=None,
-        use_stresses=True,
-        ignore_ambiguous_words=True,
-        heteronyms=None,
-        phoneme_probability: Optional[float] = None,
-        espeak_logging_level=ERROR,
+        *,
+        space=' ',
+        silence=None,
+        apostrophe=True,
+        sep='|',  # To be able to distinguish between 2/3 letters codes.
+        add_blank_at=None,
+        pad_with_space=False,
+        text_preprocessing_func=lambda text: chinese_text_preprocessing(text),
     ):
-        """IPA tokenizer using the Phonemizer library with an eSpeak backend
+        """Chinese phoneme-based tokenizer.
+        Args:
+            g2p: Grapheme to phoneme module.
+            punct: Whether to reserve grapheme for basic punctuation or not.
+            non_default_punct_list: List of punctuation marks which will be used instead default.
+            space: Space token as string.
+            silence: Silence token as string (will be disabled if it is None).
+            apostrophe: Whether to use apostrophe or not.
+            sep: Separation token as string.
+            add_blank_at: Add blank to labels in the specified order ("last") or after tokens (any non None),
+             if None then no blank in labels.
+            pad_with_space: Whether to pad text with spaces at the beginning and at the end or not.
+            text_preprocessing_func: Text preprocessing function for correct execution of the tokenizer.
+             Basically, it replaces all non-unicode characters with unicode ones.
+             Note that lower() function shouldn't be applied here, in case the text contains phonemes (it will be handled by g2p).
+        """
+        tokens = []
+        self.space, tokens = len(tokens), tokens + [space]  # Space
 
-        https://github.com/bootphon/phonemizer
-        http://espeak.sourceforge.net/
+        if silence is not None:
+            self.silence, tokens = len(tokens), tokens + [silence]  # Silence
 
-        To use this class you will need to install eSpeak separately from NeMo.
+        tokens.extend(self.PRONUNCIATION_LIST)
+        tokens.extend(self.TONES_LIST)
+        tokens.extend(string.ascii_lowercase)
 
-        For Linux: apt-get update && apt-get install espeak-ng
-        Other OS: https://github.com/espeak-ng/espeak-ng/blob/master/docs/guide.md#installation
+        if apostrophe:
+            tokens.append("'")  # Apostrophe
 
-        eSpeak provides out-of-the-box text normalization & g2p for a large number of languages.
-        This tokenizer incorporates that functionality with a phoneme dictionary that allows overriding
-        pronunciation for specific words that eSpeak may get wrong.
+        if punct:
+            if non_default_punct_list is not None:
+                self.PUNCT_LIST = non_default_punct_list
+            else:
+                self.PUNCT_LIST = list(self.ZH_PUNCT_LIST)
+            tokens.extend(self.PUNCT_LIST)
 
-        This class currently supports: English, Spanish, and German. For other languages you will have to provide
-        the character set for the language using "non_default_chars" and "non_default_punct_list".
+        super().__init__(tokens, sep=sep, add_blank_at=add_blank_at)
+
+        self.punct = punct
+        self.pad_with_space = pad_with_space
+
+        self.text_preprocessing_func = text_preprocessing_func
+        self.g2p = g2p
+
+    def encode(self, text):
+        """See base class for more information."""
+
+        text = self.text_preprocessing_func(text)
+        g2p_text = self.g2p(text)  # TODO: handle infer
+        return self.encode_from_g2p(g2p_text, text)
+
+    def encode_from_g2p(self, g2p_text: List[str], raw_text: Optional[str] = None):
+        """
+        Encodes text that has already been run through G2P.
+        Called for encoding to tokens after text preprocessing and G2P.
 
         Args:
-            language: Language string http://espeak.sourceforge.net/languages.html
-            phoneme_dict (str, Path, Dict): Path to file in CMUdict format or dictionary of CMUdict-like entries.
-                (For default behavior, provide local path to scripts/tts_dataset_files/ipa_dict-default.txt)
-            punct: Whether to keep punctuation or remove it.
-            pad_with_space: Whether to pad text with spaces at the beginning and at the end or not.
-            non_default_chars: List of characters which will be used instead of default.
-            non_default_punct_list: List of punctuation marks which will be used instead of default.
-            use_stresses: Whether to include stresses/accents in the IPA output.
-            ignore_ambiguous_words: Whether to not handle word via phoneme_dict with ambiguous phoneme sequences.
-            heteronyms (str, Path, List): Path to file with heteronyms (every line is new word) or list of words.
-            phoneme_probability (Optional[float]): The probability (0.<var<1.) that each word is phonemized for mixed
-                representation training. Defaults to None which is the same as 1.
-            espeak_logging_level: Logging level for eSpeak.
-                Defaults to 'logging.ERROR' (to avoid a lot of unhelpful warning logs).
+            g2p_text: G2P's output, could be a mixture of Chinese phonemes and English letters.
+            raw_text: original raw input
         """
+        ps, space, tokens = [], self.tokens[self.space], set(self.tokens)
+        for p in g2p_text:  # noqa
+            # Add space if last one isn't one
+            if p == space and len(ps) > 0 and ps[-1] != space:
+                ps.append(p)
+            # Add next phoneme or char (if chars=True)
+            elif (p.isalnum() or p == "'" or p in self.PRONUNCIATION_LIST or p in self.TONES_LIST) and p in tokens:
+                ps.append(p)
+            # Add punct
+            elif (p in self.PUNCT_LIST) and self.punct:
+                ps.append(p)
+            # Warn about unknown char/phoneme
+            elif p != space:
+                message = f"Text: [{''.join(g2p_text)}] contains unknown char/phoneme: [{p}]."
+                if raw_text is not None:
+                    message += f"Original text: [{raw_text}]. Symbol will be skipped."
+                logging.warning(message)
 
-        if not (ctypes.util.find_library('espeak-ng') or ctypes.util.find_library('espeak')):
-            raise ImportError(
-                "PhonemizerTokenizer requires eSpeak to be installed. "
-                "Please follow the instructions at: "
-                "https://github.com/espeak-ng/espeak-ng/blob/master/docs/guide.md#installation"
-            )
+        # Remove trailing spaces
+        if ps:
+            while ps[-1] == space:
+                ps.pop()
 
-        use_chars = phoneme_probability is not None
-        if non_default_chars:
-            char_set = non_default_chars
-        else:
-            locale = self._get_locale(language)
-            char_set = get_ipa_character_set(locale)
-            if use_stresses:
-                char_set.update(self.ESPEAK_SYMBOLS)
-            if use_chars:
-                grapheme_set = get_grapheme_character_set(locale)
-                char_set.update(grapheme_set)
+        if self.pad_with_space:
+            ps = [space] + ps + [space]
 
-        if not punct:
-            punct_list = []
-        elif non_default_punct_list:
-            punct_list = non_default_punct_list
-        else:
-            locale = self._get_locale(language)
-            punct_list = get_ipa_punctuation_list(locale)
-
-        espeak_logger = getLogger('espeak')
-        espeak_logger.setLevel(espeak_logging_level)
-        self.espeak_backend = EspeakBackend(
-            language=language, preserve_punctuation=punct, with_stress=use_stresses, logger=espeak_logger
-        )
-
-        ipa_g2p = IPAG2P(
-            phoneme_dict=phoneme_dict,
-            apply_to_oov_word=self._text_to_phonemes_espeak,
-            word_tokenize_func=ipa_word_tokenize,
-            ignore_ambiguous_words=ignore_ambiguous_words,
-            heteronyms=heteronyms,
-            use_chars=use_chars,
-            phoneme_probability=phoneme_probability,
-            use_stresses=use_stresses,
-            set_graphemes_upper=True,
-        )
-        # Merge chars with all symbols found in the phoneme dictionary.
-        ipa_g2p.add_symbols(char_set)
-
-        super().__init__(
-            g2p=ipa_g2p,
-            punct=punct,
-            non_default_punct_list=punct_list,
-            pad_with_space=pad_with_space,
-            text_preprocessing_func=lambda text: text,
-        )
-
-    @staticmethod
-    def _get_locale(language):
-        # Converts eSpeak language string to ISO language tag
-        if language == "en-us":
-            return "en-US"
-        elif language == "es":
-            return "es-ES"
-        elif language == "de":
-            return "de-DE"
-
-        raise ValueError(f"Language not supported {language}")
-
-    def _text_to_phonemes_espeak(self, text):
-        phonemes = self.espeak_backend.phonemize([text])[0]
-        phonemes = phonemes.strip()
-        return phonemes
-
-    def text_to_phonemes(self, text):
-        tokens = self.encode(text)
-        phoneme_string = self.decode(tokens)
-        phonemes = phoneme_string.replace(self.sep, '')
-        return phonemes
+        return [self._token2id[p] for p in ps]
