@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import enum
 import math
 
 from omegaconf.dictconfig import DictConfig
@@ -26,6 +27,11 @@ from nemo.utils import logging
 __all__ = ["MegatronT5Model"]
 
 
+class T5Sentinel(enum.Enum):
+    FIRST = '<extra_id_0>'
+    END = '<extra_id_1>'
+
+
 class MegatronT5Model(MegatronLMEncoderDecoderModel):
     """
     Megatron T5 pretraining
@@ -81,72 +87,119 @@ class MegatronT5Model(MegatronLMEncoderDecoderModel):
             tokenizer=self.tokenizer,
             tokenizer_cfg=self._cfg.tokenizer,
             dataset_type=self._cfg.data.get("dataset_type", "t5"),
+            add_sentinel_tokens_in_reverse_order=self._cfg.tokenizer.get(
+                "add_sentinel_tokens_in_reverse_order", False
+            ),
+            add_sentinel_tokens_first=self._cfg.tokenizer.get("add_sentinel_tokens_first", False),
         )
         super()._build_vocab()
 
     @classmethod
-    def add_special_tokens_to_tokenizer(cls, tokenizer, tokenizer_cfg, dataset_type="t5"):
-        # T5-related construction
-        if tokenizer_cfg.library == 'huggingface' or tokenizer_cfg.library == 'megatron':
-            additional_tokens = {
-                'additional_special_tokens': [f'<extra_id_{i}>' for i in range(tokenizer_cfg.num_sentinel_tokens)]
-            }
-            if dataset_type == "ul2":
-                for mask_type in ['r', 's', 'x']:
-                    additional_tokens['additional_special_tokens'].extend([f'<extra_id_{mask_type}>'])
-            tokenizer.add_special_tokens(additional_tokens)
+    def _add_sentinel_tokens(cls, tokenizer, num_sentinel_tokens, add_sentinel_tokens_in_reverse_order):
+        # Special check to see if <extra_id_{}> is already present in the tokenizer. If it is, only modify the additional_special_tokens function.
+        for i in range(num_sentinel_tokens):
+            if add_sentinel_tokens_in_reverse_order:
+                i = num_sentinel_tokens - i - 1
+            if len(tokenizer.text_to_ids(f'<extra_id_{i}>')) == 1:
+                tokenizer.special_token_to_id[f'<extra_id_{i}>'] = tokenizer.text_to_ids(f'<extra_id_{i}>')[0]
+            else:
+                tokenizer.add_special_tokens([f'<extra_id_{i}>'])
 
-        if tokenizer_cfg.library == 'sentencepiece':
-            # Need to add cls, sep, mask tokens to the tokenizer if they don't exist.
-            # If cls, sep and mask are not attributes of the tokenizer, add it.
-            if not hasattr(tokenizer, 'cls_token'):
-                tokenizer.add_special_tokens({'cls_token': '<cls>'})
-            if not hasattr(tokenizer.tokenizer, 'sep_id'):
-                tokenizer.add_special_tokens({'sep_token': '<sep>'})
-            if not hasattr(tokenizer.tokenizer, 'mask_id'):
-                tokenizer.add_special_tokens({'mask_token': '<mask>'})
+    @classmethod
+    def _add_base_special_tokens(cls, tokenizer, is_huggingface_converted_model):
+        # Need to add cls, sep, mask tokens to the tokenizer if they don't exist.
+        # If cls, sep and mask are not attributes of the tokenizer, add it.
+        if not hasattr(tokenizer, 'cls_token'):
+            tokenizer.add_special_tokens({'cls_token': '<cls>'})
+        if not hasattr(tokenizer.tokenizer, 'sep_id'):
+            tokenizer.add_special_tokens({'sep_token': '<sep>'})
+        if not hasattr(tokenizer.tokenizer, 'mask_id'):
+            tokenizer.add_special_tokens({'mask_token': '<mask>'})
 
-            # bos, eos, pad and unk may be present in the provided spm .model file, if they are, use it.
-            if not hasattr(tokenizer, 'pad_token'):
-                if hasattr(tokenizer.tokenizer, 'pad_id') and tokenizer.tokenizer.pad_id() > 0:
+        # bos, eos, pad and unk may be present in the provided spm .model file, if they are, use it.
+        if not hasattr(tokenizer, 'pad_token'):
+            # TODO: Figure out how to do backward compat with pad_id > 0 and >= 0.
+            if is_huggingface_converted_model:
+                if hasattr(tokenizer.tokenizer, 'pad_id') and tokenizer.tokenizer.pad_id() >= 0:
                     tokenizer.pad_token = tokenizer.tokenizer.id_to_piece(tokenizer.tokenizer.pad_id())
                 else:
                     tokenizer.add_special_tokens({'pad_token': '<pad>'})
             else:
-                tokenizer.add_special_tokens({'pad_token': '<pad>'})
-
-            if not hasattr(tokenizer, 'bos_token'):
-                if hasattr(tokenizer.tokenizer, 'bos_id') and tokenizer.tokenizer.bos_id() > 0:
-                    tokenizer.bos_token = tokenizer.tokenizer.id_to_piece(tokenizer.tokenizer.bos_id())
+                if hasattr(tokenizer.tokenizer, 'pad_id') and tokenizer.tokenizer.pad_id() > 0:
+                    tokenizer.pad_token = tokenizer.tokenizer.id_to_piece(tokenizer.tokenizer.pad_id())
                 else:
-                    tokenizer.add_special_tokens({'bos_token': '<bos>'})
+                    tokenizer.add_special_tokens({'pad_token': '<pad>'})
+        else:
+            tokenizer.add_special_tokens({'pad_token': '<pad>'})
+
+        if not hasattr(tokenizer, 'bos_token'):
+            if hasattr(tokenizer.tokenizer, 'bos_id') and tokenizer.tokenizer.bos_id() > 0:
+                tokenizer.bos_token = tokenizer.tokenizer.id_to_piece(tokenizer.tokenizer.bos_id())
             else:
-                tokenizer.add_special_tokens({'bos_token': '<s>'})
+                tokenizer.add_special_tokens({'bos_token': '<bos>'})
+        else:
+            tokenizer.add_special_tokens({'bos_token': '<s>'})
 
-            if not hasattr(tokenizer, 'eos_token'):
-                if hasattr(tokenizer.tokenizer, 'eos_id') and tokenizer.tokenizer.eos_id() > 0:
-                    tokenizer.eos_token = tokenizer.tokenizer.id_to_piece(tokenizer.tokenizer.eos_id())
-                else:
-                    tokenizer.add_special_tokens({'eos_token': '<eos>'})
+        if not hasattr(tokenizer, 'eos_token'):
+            if hasattr(tokenizer.tokenizer, 'eos_id') and tokenizer.tokenizer.eos_id() > 0:
+                tokenizer.eos_token = tokenizer.tokenizer.id_to_piece(tokenizer.tokenizer.eos_id())
             else:
-                tokenizer.add_special_tokens({'eos_token': '</s>'})
+                tokenizer.add_special_tokens({'eos_token': '<eos>'})
+        else:
+            tokenizer.add_special_tokens({'eos_token': '</s>'})
 
-            # Special check to see if <extra_id_{}> is already present in the tokenizer. If it is, only modify the additional_special_tokens function.
-            for i in range(tokenizer_cfg.num_sentinel_tokens):
-                if f'▁<extra_id_{i}>' in tokenizer.vocab:
-                    tokenizer.special_token_to_id[f'<extra_id_{i}>'] = tokenizer.text_to_ids(f'<extra_id_{i}>')[0]
-                else:
-                    tokenizer.add_special_tokens([f'<extra_id_{i}>'])
+    @classmethod
+    def add_special_tokens_to_tokenizer(
+        cls,
+        tokenizer,
+        tokenizer_cfg,
+        dataset_type="t5",
+        add_sentinel_tokens_in_reverse_order=False,
+        add_sentinel_tokens_first=False,
+    ):
+        # T5-related construction
+        if tokenizer_cfg.library == 'huggingface' or tokenizer_cfg.library == 'megatron':
+            additional_tokens = {
+                'additional_special_tokens': [
+                    f'<extra_id_{i}>' for i in range(tokenizer_cfg.get('num_sentinel_tokens', 0))
+                ]
+            }
+            if dataset_type == "ul2":
+                mask_types = ['r', 's', 'x']
+                for mask_type in mask_types:
+                    additional_tokens['additional_special_tokens'].extend([f'<extra_id_{mask_type}>'])
+            if additional_tokens['additional_special_tokens']:
+                tokenizer.add_special_tokens(additional_tokens)
+
+        if tokenizer_cfg.library == 'sentencepiece':
+            # NOTE: This is an ugly way to support both NeMo-Megatron trained checkpoints and huggingface checkpoints.
+            # Huggingface and Google checkpoints will add sentinel tokens first (right after the base vocabulary), but in NeMo-Megatron, we add <cls>, <sep>, <mask>, <pad>, <bos> etc. beofore sentinel tokens <extra_id_xx>.
+            if add_sentinel_tokens_first:
+                if tokenizer_cfg.get('num_sentinel_tokens', 0) > 0:
+                    cls._add_sentinel_tokens(
+                        tokenizer, tokenizer_cfg.num_sentinel_tokens, add_sentinel_tokens_in_reverse_order
+                    )
+                cls._add_base_special_tokens(tokenizer, is_huggingface_converted_model=True)
+            else:
+                cls._add_base_special_tokens(tokenizer, is_huggingface_converted_model=False)
+                if tokenizer_cfg.get('num_sentinel_tokens', 0) > 0:
+                    cls._add_sentinel_tokens(
+                        tokenizer, tokenizer_cfg.num_sentinel_tokens, add_sentinel_tokens_in_reverse_order
+                    )
 
             if dataset_type == "ul2":
                 for mask_type in ['r', 's', 'x']:
-                    if f'▁<extra_id_{mask_type}>' in tokenizer.vocab:
-                        tokenizer.special_token_to_id[f'<extra_id_{i}>'] = tokenizer.text_to_ids(f'<extra_id_{i}>')[0]
+                    if len(tokenizer.text_to_ids(f'▁<extra_id_{mask_type}>')) == 1:
+                        tokenizer.special_token_to_id[f'<extra_id_{mask_type}>'] = tokenizer.text_to_ids(
+                            f'<extra_id_{mask_type}>'
+                        )[0]
                     else:
                         tokenizer.add_special_tokens([f'<extra_id_{mask_type}>'])
 
     def build_train_valid_test_datasets(self):
         logging.info(f'Building {self.model_name} datasets.')
+        if self.trainer.limit_val_batches > 1.0 and isinstance(self.trainer.limit_val_batches, float):
+            raise ValueError("limit_val_batches must be an integer or float less than or equal to 1.0.")
         global_batch_size = self._cfg.global_batch_size
         eval_iters = (self.trainer.max_steps // self.trainer.val_check_interval + 1) * self.trainer.limit_val_batches
         test_iters = self.trainer.limit_test_batches
@@ -155,6 +208,12 @@ class MegatronT5Model(MegatronLMEncoderDecoderModel):
             eval_iters * global_batch_size,
             test_iters * global_batch_size,
         ]
+
+        if self.trainer.limit_val_batches <= 1.0 and isinstance(self.trainer.limit_val_batches, float):
+            train_valid_test_num_samples[
+                1
+            ] = 1  # This is to make sure we only have one epoch on every validation iteration
+
         self._train_ds, self._validation_ds, self._test_ds = build_train_valid_test_datasets(
             cfg=self._cfg,
             trainer=self.trainer,
