@@ -21,22 +21,26 @@ from omegaconf.dictconfig import DictConfig
 from omegaconf.omegaconf import open_dict
 from pytorch_lightning.trainer.trainer import Trainer
 
-from nemo.collections.nlp.data.language_modeling.megatron.t5_prompt_learning_dataset import (
-    T5PromptLearningDataset,
-    T5Sentinel,
-)
+from nemo.collections.nlp.data.language_modeling.megatron.t5_prompt_learning_dataset import T5PromptLearningDataset
 from nemo.collections.nlp.models.language_modeling.megatron_base_prompt_learning_model import (
     MegatronBasePromptLearningModel,
 )
+from nemo.collections.nlp.models.language_modeling.megatron_finetune_model import MegatronT5FinetuneModel
 from nemo.collections.nlp.models.language_modeling.megatron_t5_model import MegatronT5Model
+from nemo.collections.nlp.modules.common import VirtualPromptSource
 from nemo.collections.nlp.modules.common.megatron.utils import average_losses_across_data_parallel_group
 from nemo.collections.nlp.parts.nlp_overrides import NLPSaveRestoreConnector
 from nemo.collections.nlp.parts.utils_funcs import get_last_rank
-from nemo.utils import logging
+from nemo.utils import AppState, logging
 
 try:
     from apex.transformer import parallel_state
+    from apex.transformer.enums import ModelType
     from apex.transformer.pipeline_parallel.schedules.fwd_bwd_no_pipelining import forward_backward_no_pipelining
+    from apex.transformer.pipeline_parallel.schedules.fwd_bwd_pipelining_without_interleaving import (
+        forward_backward_pipelining_without_interleaving,
+    )
+    from apex.transformer.pipeline_parallel.utils import _reconfigure_microbatch_calculator, get_micro_batch_size
 
     HAVE_APEX = True
 
@@ -69,6 +73,12 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
 
     def __init__(self, cfg: DictConfig, trainer: Trainer):
         super().__init__(cfg, trainer)
+        self.model_type = ModelType.encoder_and_decoder
+
+    def first_stage_of_pipeline(self):
+        if self.frozen_model.enc_dec_model.pre_process and parallel_state.get_pipeline_model_parallel_rank() == 0:
+            return True
+        return False
 
     def forward(
         self, input_ids, dec_input, enc_mask, dec_mask, position_ids, taskname_ids, labels=None, inference=False,
@@ -77,18 +87,29 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
         Special forward method for p-tuning/prompt-tuning pretrained
         T5 style models.
         """
-        # Get embeddings for text tokens and insert virtual token embeddings
-        if inference:
-            input_embeds = self.embed_input_inference(input_ids, taskname_ids)
-        else:
-            input_embeds = self.embed_input_train(input_ids, taskname_ids)
+        batch_size, seq_length = input_ids.shape
 
-        # TODO: This check needs to be revisited with PP support.
-        if hasattr(self.frozen_model.enc_dec_model.encoder_embedding, 'position_embeddings'):
-            position_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.position_embeddings(position_ids)
-            encoder_input = input_embeds + position_embeddings
+        if self.first_stage_of_pipeline():
+            # Get embeddings for text tokens and insert virtual token embeddings
+            if inference:
+                input_embeds = self.embed_input_inference(input_ids, taskname_ids)
+            else:
+                input_embeds = self.embed_input_train(input_ids, taskname_ids)
+
+            # TODO: This check needs to be revisited with PP support.
+            if hasattr(self.frozen_model.enc_dec_model.encoder_embedding, 'position_embeddings'):
+                position_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.position_embeddings(
+                    position_ids
+                )
+                encoder_input = input_embeds + position_embeddings
+            else:
+                encoder_input = input_embeds
         else:
-            encoder_input = input_embeds
+            encoder_input = None
+
+        # If the decoder input starts with <pad> instead of <bos>, which is the case for huggingface T5 models, we don't want to mask the first token.
+        # For NeMo-Megatron, the sequence starts with <bos>, which is never masked so we can always set index 0 to be unmasked.
+        dec_mask[:, 0] = 1
 
         # Call forward on T5 model with preprocessed embeddings
         if self.autocast_dtype == torch.float32:
@@ -143,8 +164,24 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
             save_restore_connector=NLPSaveRestoreConnector(),
         )
 
-        # Freeze all T5 model weights for prompt-tuning/p-tuning
-        self.frozen_model.freeze()
+    def setup_optimizer_param_groups(self):
+        """
+        ModelPT override. Optimizer will get self._optimizer_param_groups. 
+        Only want virtual prompt params to be passed to the optimizer.
+        """
+        ## Freeze frozen model
+        for param in self.frozen_model.parameters():
+            param.requires_grad = False
+
+        virtual_prompt_params = {'params': []}
+
+        if self.first_stage_of_pipeline():
+            virtual_prompt_params['params'].extend([param for param in self.prompt_table.parameters()])
+
+            if self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER:
+                virtual_prompt_params['params'].extend([param for param in self.prompt_encoder.parameters()])
+
+        self._optimizer_param_groups = (virtual_prompt_params,)
 
     def fwd_bwd_step(self, batch, batch_idx, forward_only):
         """
@@ -153,10 +190,22 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
         """
         # Get seq length of batch
         _, seq_length = batch[0].shape
-        tensor_shape = [seq_length, self.cfg.micro_batch_size, self.hidden_size]
+        _, dec_seq_length = batch[1].shape
+        tensor_shape = [seq_length, get_micro_batch_size(), self.hidden_size]
 
         if self.cfg.get('pipeline_model_parallel_size', 1) > 1:
-            raise Exception("Pipeline parallelism is not supported yet")
+            losses_reduced_per_micro_batch = forward_backward_pipelining_without_interleaving(
+                forward_step_func=self.get_forward_output_and_loss_func(),
+                batch=batch,
+                model=self,
+                forward_only=forward_only,
+                tensor_shape=tensor_shape,
+                decoder_sequence_length=dec_seq_length,
+                dtype=self.autocast_dtype,
+                grad_scaler=self.trainer.precision_plugin.scaler if self.cfg.precision == 16 else None,
+                sequence_parallel_enabled=False,
+                sync_batch_comm=self.frozen_model.cfg.get('sync_batch_comm', False),
+            )
         else:
             losses_reduced_per_micro_batch = forward_backward_no_pipelining(
                 forward_step_func=self.get_forward_output_and_loss_func(),
@@ -164,9 +213,10 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
                 model=self,
                 forward_only=forward_only,
                 tensor_shape=tensor_shape,
+                decoder_sequence_length=dec_seq_length,
                 dtype=self.autocast_dtype,
-                disable_autocast=True,
                 grad_scaler=self.trainer.precision_plugin.scaler if self.cfg.precision == 16 else None,
+                sync_batch_comm=self.frozen_model.cfg.get('sync_batch_comm', False),
             )
 
         # only the last stages of the pipeline return losses
@@ -189,6 +239,7 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
             output_tensor, encoder_input = model(
                 enc_input, dec_input, enc_mask, dec_mask, position_ids, taskname_ids, labels, inference=False
             )
+            output_tensor = output_tensor.contiguous()
 
             def loss_func(output_tensor):
                 loss = self.frozen_model.loss_func(loss_mask, output_tensor)
@@ -212,8 +263,38 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
         """
         return
 
-    def training_step(self, batch, batch_idx):
+    def set_input_tensor(self, input_tensor):
+        """Set input tensor to be used instead of forward()'s input.
+        When using pipeline parallelism the input from the previous
+        stage comes from communication, not from the input, so the
+        model's forward_step_func won't have it. This function is thus
+        used by internal code to bypass the input provided by the
+        forward_step_func"""
+        self.frozen_model.enc_dec_model.set_input_tensor(input_tensor)
 
+    def _reconfigure_batch_sizes(self, gbs: int, mbs: int):
+        app_state = AppState()
+        _reconfigure_microbatch_calculator(
+            rank=app_state.global_rank,
+            rampup_batch_size=None,
+            global_batch_size=gbs,
+            micro_batch_size=mbs,
+            data_parallel_size=parallel_state.get_data_parallel_world_size(),
+        )
+
+    def on_train_epoch_start(self) -> None:
+        gbs = self.cfg.global_batch_size
+        mbs = self.cfg.micro_batch_size
+        self._reconfigure_batch_sizes(gbs, mbs)
+        return super().on_train_epoch_start()
+
+    def on_validation_epoch_start(self) -> None:
+        gbs = self.cfg.get('validation_global_batch_size', self.cfg.global_batch_size)
+        mbs = self.cfg.get('validation_micro_batch_size', self.cfg.micro_batch_size)
+        self._reconfigure_batch_sizes(gbs, mbs)
+        return super().on_validation_epoch_start()
+
+    def training_step(self, batch, batch_idx):
         self._optimizer.zero_grad()
         loss_mean = self.fwd_bwd_step(batch, batch_idx, forward_only=False)
         self.allreduce_gradients()
@@ -223,7 +304,7 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
         # we can avoid this broadcast by updating the PTL log function to accept specific ranks
         torch.distributed.broadcast(loss_mean, get_last_rank())
 
-        if self.cfg.precision == 16:
+        if self.cfg.precision == 16 and hasattr(self.trainer.precision_plugin.scaler, "_scale"):
             loss_scale = self.trainer.precision_plugin.scaler._scale
             if loss_scale is not None:
                 self.log('loss_scale', loss_scale)
@@ -232,73 +313,85 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
         lr = self._optimizer.param_groups[0]['lr']
         self.log('lr', lr, rank_zero_only=True)
         self.log('global_step', self.trainer.global_step, prog_bar=True, rank_zero_only=True)
-
         return loss_mean
 
-    def inference_step(self, batch, batch_idx, inference=False):
-        enc_input, dec_input, labels, loss_mask, enc_mask, dec_mask, position_ids, taskname_ids = batch
+    def _reconfigure_and_process_inference_batch(self, global_batch_size_per_gpu, gbs):
+        # This should happen only on the last batch of the dataset.
+        if global_batch_size_per_gpu != gbs // parallel_state.get_data_parallel_world_size():
+            # NOTE: This is reconfiguring to make sure there is no grad-acc for validation batches.
+            app_state = AppState()
+            _reconfigure_microbatch_calculator(
+                rank=app_state.global_rank,
+                rampup_batch_size=None,
+                global_batch_size=global_batch_size_per_gpu * parallel_state.get_data_parallel_world_size(),
+                micro_batch_size=global_batch_size_per_gpu,
+                data_parallel_size=parallel_state.get_data_parallel_world_size(),
+            )
+
+    def validation_step(self, batch, batch_idx, inference=False):
+        input_ids, dec_input, labels, loss_mask, enc_mask, dec_mask, position_ids, taskname_ids = batch
 
         mode = self.training
         self.eval()
-
-        input_embeds = self.embed_input_train(enc_input, taskname_ids)
-
-        # TODO: This check needs to be revisited with PP support.
-        if hasattr(self.frozen_model.enc_dec_model.encoder_embedding, 'position_embeddings'):
-            position_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.position_embeddings(position_ids)
-            encoder_input = input_embeds + position_embeddings
-        else:
-            encoder_input = input_embeds
-
+        gbs = self.cfg.get('validation_global_batch_size', self.cfg.global_batch_size)
+        self._reconfigure_and_process_inference_batch(input_ids.size(0), gbs)
         loss_mean = self.fwd_bwd_step(batch, batch_idx, forward_only=True)
 
+        if self.first_stage_of_pipeline():
+            # Get embeddings for text tokens and insert virtual token embeddings
+            input_embeds = self.embed_input_train(input_ids, taskname_ids)
+
+            if hasattr(self.frozen_model.enc_dec_model.encoder_embedding, 'position_embeddings'):
+                position_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.position_embeddings(
+                    position_ids
+                )
+                encoder_input = input_embeds + position_embeddings
+            else:
+                encoder_input = input_embeds
+        else:
+            encoder_input = None
+
         predicted_token_ids, log_probs = self.frozen_model.decode(
-            tokens_enc=enc_input,
+            tokens_enc=input_ids,
             enc_mask=enc_mask,
             num_tokens_to_generate=self.decoder_seq_length,
             encoder_input=encoder_input,
+            bos_id=self.tokenizer.pad_id
+            if self.cfg.data.get('decoder_starts_with_pad', False)
+            else self.tokenizer.bos_id,
         )
-
-        processed_inputs, processed_preds, processed_labels = [], [], []
-        preds = predicted_token_ids.cpu().numpy().tolist()
-        labels = labels.cpu().numpy().tolist()
-        enc_inputs = enc_input.cpu().numpy().tolist()
-
-        for i, (enc_input, pred, label) in enumerate(zip(enc_inputs, preds, labels)):
-            if self.tokenizer.eos_id in pred:
-                idx = pred.index(self.tokenizer.eos_id)
-                pred = pred[:idx]
-
-            # Sentencepiece case
-            if hasattr(self.tokenizer, 'special_token_to_id'):
-                pred = [id for id in pred if id not in self.tokenizer.special_token_to_id.values()]
-                label = [id for id in label if id not in self.tokenizer.special_token_to_id.values()]
-                enc_input = [id for id in enc_input if id not in self.tokenizer.special_token_to_id.values()]
-            # HF Autotokenizer case.
-            else:
-                pred = [id for id in pred if id not in self.tokenizer.tokenizer.additional_special_tokens_ids]
-                label = [id for id in label if id not in self.tokenizer.tokenizer.additional_special_tokens_ids]
-                enc_input = [
-                    id for id in enc_input if id not in self.tokenizer.tokenizer.additional_special_tokens_ids
-                ]
-
-            pred = self.tokenizer.ids_to_text(pred)
-            label = self.tokenizer.ids_to_text(label)
-            enc_input = self.tokenizer.ids_to_text(enc_input)
-
-            processed_preds.append(pred)
-            processed_labels.append(label)
-            processed_inputs.append(enc_input)
+        # Special ids to text function to handle stripping <eos> and special tokens with sentencepiece tokenizers.
+        preds_text = MegatronT5FinetuneModel.ids_to_text(predicted_token_ids, self.tokenizer)
+        labels_text = MegatronT5FinetuneModel.ids_to_text(labels, self.tokenizer)
+        input_text = MegatronT5FinetuneModel.ids_to_text(input_ids, self.tokenizer)
 
         self.train(mode=mode)
+        self.frozen_model.eval()
         return {
             'loss': loss_mean,
-            'predicted_token_ids': processed_preds,
-            'labels': processed_labels,
-            'enc_inputs': processed_inputs,
+            'predicted_token_ids': preds_text,
+            'labels': labels_text,
+            'enc_inputs': input_text,
         }
 
-    def inference_epoch_end(self, outputs):
+    def validation_epoch_end(self, outputs):
+        if self.cfg.get('pipeline_model_parallel_size', 1) > 1:
+            if parallel_state.is_pipeline_last_stage():
+                # only the last pipeline parallel stages return loss
+                averaged_loss = torch.stack([i['loss'] for i in outputs]).mean()
+            else:
+                averaged_loss = torch.tensor(0.0).cuda()
+
+            # we can only log on one rank if it is rank zero so we broadcast from last rank
+            torch.distributed.broadcast(averaged_loss, get_last_rank())
+
+            self.log('val_loss', averaged_loss, prog_bar=True, rank_zero_only=True)
+            logging.info(f'Validation loss: {averaged_loss}')
+
+        else:
+            averaged_loss = torch.stack([item['loss'] for item in outputs]).mean()
+            logging.info(f'Validation loss: {averaged_loss}')
+            self.log('val_loss', averaged_loss, prog_bar=True, rank_zero_only=True)
 
         gather_results = [None for _ in range(parallel_state.get_data_parallel_world_size())]
 
@@ -333,18 +426,18 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
         else:
             val_acc = torch.tensor(0.0).cuda()
 
-        averaged_loss = torch.stack([item['loss'] for item in outputs]).mean()
-        logging.info(f'Validation loss: {averaged_loss}')
-
-        self.log('val_loss', averaged_loss, prog_bar=True, rank_zero_only=True)
         self.log('val_acc', val_acc, prog_bar=True, rank_zero_only=True)
+        logging.info(f'val_loss: {averaged_loss}')
 
-    def validation_step(self, batch, batch_idx, inference=False):
-        outcome = self.inference_step(batch, batch_idx, inference=inference)
-        return outcome
+        # Save inference ready .nemo checkpoint version
+        if self.cfg.get("save_nemo_on_validation_end", False):
+            if self.lowest_val_loss is None or averaged_loss < self.lowest_val_loss:
+                self.save_checkpoint_as_nemo_file()
+                self.lowest_val_loss = averaged_loss
 
-    def validation_epoch_end(self, outputs):
-        self.inference_epoch_end(outputs)
+        gbs = self.cfg.global_batch_size
+        mbs = self.cfg.micro_batch_size
+        self._reconfigure_batch_sizes(gbs, mbs)
 
     def test_step(self, batch, batch_idx):
         return self.validation_step(batch, batch_idx)
@@ -366,13 +459,17 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
             min_seq_length=self.cfg.data.get('min_seq_length', 1),
             add_bos=self.cfg.data.get('add_bos', False),
             add_eos=self.cfg.data.get('add_eos', True),
+            decoder_starts_with_pad=self.cfg.data.get('decoder_starts_with_pad', False),
+            add_eos_to_decoder_output=self.cfg.data.get('add_eos_to_decoder_output', True),
+            add_sentinel_to_input=self.cfg.data.get('add_sentinel_to_input', True),
+            ul2_prompt_token=self.cfg.data.get('ul2_prompt_token', None),
             for_train=for_train,
         )
 
         rank = parallel_state.get_data_parallel_rank()
         world_size = parallel_state.get_data_parallel_world_size()
         sampler = torch.utils.data.distributed.DistributedSampler(
-            dataset, num_replicas=world_size, rank=rank, shuffle=shuffle
+            dataset, num_replicas=world_size, rank=rank, shuffle=shuffle, seed=self.cfg.seed
         )
 
         dataloader = torch.utils.data.DataLoader(
@@ -383,91 +480,61 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
             drop_last=drop_last,
             num_workers=num_workers,
             pin_memory=pin_memory,
+            persistent_workers=True,  # (@adithyare and @eharper) We need to set this to True to get around issues with spawn=True
         )
         print('build success', len(dataloader), dataset_paths)
         return dataset, dataloader
 
     def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: int = 0) -> Any:
 
-        enc_input, dec_input, labels, loss_mask, enc_mask, dec_mask, position_ids, taskname_ids = batch
+        input_ids, dec_input, labels, loss_mask, enc_mask, dec_mask, position_ids, taskname_ids = batch
 
-        input_embeds = self.embed_input_inference(enc_input, taskname_ids)
+        batch_size, seq_length = input_ids.shape
+        if self.first_stage_of_pipeline():
+            input_embeds = self.embed_input_inference(input_ids, taskname_ids)
+
+            # TODO: This check needs to be revisited with PP support.
+            if hasattr(self.frozen_model.enc_dec_model.encoder_embedding, 'position_embeddings'):
+                position_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.position_embeddings(
+                    position_ids
+                )
+                encoder_input = input_embeds + position_embeddings
+            else:
+                encoder_input = input_embeds
 
-        # TODO: This check needs to be revisited with PP support.
-        if hasattr(self.frozen_model.enc_dec_model.encoder_embedding, 'position_embeddings'):
-            position_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.position_embeddings(position_ids)
-            encoder_input = input_embeds + position_embeddings
         else:
-            encoder_input = input_embeds
+            encoder_input = torch.zeros((batch_size, seq_length, self.hidden_size), dtype=self.autocast_dtype).cuda()
 
         predicted_token_ids, log_probs = self.frozen_model.decode(
-            tokens_enc=enc_input,
+            tokens_enc=input_ids,
             enc_mask=enc_mask,
             num_tokens_to_generate=self.decoder_seq_length,
             encoder_input=encoder_input,
+            bos_id=self.tokenizer.pad_id
+            if self.cfg.data.get('decoder_starts_with_pad', False)
+            else self.tokenizer.bos_id,
         )
-
-        processed_preds = []
-        processed_labels = []
-        processed_inputs = []
-
-        preds = predicted_token_ids.cpu().numpy().tolist()
-        enc_inputs = enc_input.cpu().numpy().tolist()
+        # Special ids to text function to handle stripping <eos> and special tokens with sentencepiece tokenizers.
+        preds_text = MegatronT5FinetuneModel.ids_to_text(predicted_token_ids, self.tokenizer)
+        input_text = MegatronT5FinetuneModel.ids_to_text(input_ids, self.tokenizer)
 
         if labels is not None:
-            labels = labels.cpu().numpy().tolist()
+            labels_text = MegatronT5FinetuneModel.ids_to_text(labels, self.tokenizer)
         else:
-            labels = [None] * len(preds)
-
-        for i, (enc_input, pred, label) in enumerate(zip(enc_inputs, preds, labels)):
-            if self.tokenizer.eos_id in pred:
-                idx = pred.index(self.tokenizer.eos_id)
-                pred = pred[:idx]
-
-            special_token_ids = (
-                self.tokenizer.special_token_to_id.values()
-                if hasattr(self.tokenizer, 'special_token_to_id')
-                else self.tokenizer.tokenizer.additional_special_tokens_ids
-            )
-            pred = [
-                id
-                for id in pred
-                if id not in special_token_ids and id not in self.tokenizer.text_to_ids(T5Sentinel.FIRST.value)
-            ]  # delete the sentinel token at the beginning of prediction
-
-            pred = self.tokenizer.ids_to_text(pred)
-            processed_preds.append(pred)
-
-            enc_input = [
-                id for id in enc_input if id not in self.tokenizer.text_to_ids(T5Sentinel.FIRST.value)
-            ]  # delete the sentinel token added to the end of input
-
-            input = self.tokenizer.ids_to_text(enc_input)
-            processed_inputs.append(input)
-
-            if label:
-                label = [
-                    id
-                    for id in label
-                    if id not in special_token_ids and id not in self.tokenizer.text_to_ids(T5Sentinel.FIRST.value)
-                ]  # delete the sentinel token at the beginning of label
-
-                label = self.tokenizer.ids_to_text(label)
-            processed_labels.append(label)
+            labels_text = [None] * len(preds_text)
 
         return {
-            'enc_input': processed_inputs,
-            'predicted_token_ids': processed_preds,
-            'log_probs': log_probs,
-            'labels': processed_labels,
+            'input_text': input_text,
+            'preds_text': preds_text,
+            'labels_text': labels_text,
         }
 
     def on_predict_epoch_end(self, outputs: List[Any]) -> None:
 
         gather_results = [None for _ in range(parallel_state.get_data_parallel_world_size())]
-        all_preds = list(itertools.chain(*[item['predicted_token_ids'] for item in outputs[0]]))
-        all_labels = list(itertools.chain(*[item['labels'] for item in outputs[0]]))
-        all_inputs = list(itertools.chain(*[item['enc_input'] for item in outputs[0]]))
+        all_preds = list(itertools.chain(*[item['preds_text'] for item in outputs[0]]))
+        all_labels = list(itertools.chain(*[item['labels_text'] for item in outputs[0]]))
+        all_inputs = list(itertools.chain(*[item['input_text'] for item in outputs[0]]))
 
         assert len(all_preds) == len(all_labels)
         assert len(all_preds) == len(all_inputs)
@@ -492,7 +559,5 @@ class MegatronT5PromptLearningModel(MegatronBasePromptLearningModel):
                         correct += 1
 
             acc = correct / len(gather_results_dedup) if all_labels[0] else None
-
-            results = {'input_prediction_pair': input_prediction_pair, 'acc': acc}
-            logging.info(f'Prediction results: {results}')
-            logging.info(f'Test finish---------------------------------')
+            logging.info(f'Prediction results: {acc}')
+            logging.info(f'Test finish')
