@@ -59,21 +59,34 @@ def get_language_model(
     init_method_std=0.02,
     use_cpu_initialization=False,
     hidden_dropout=0.1,
+    attention_dropout=0.1,
     precision=16,
     fp32_residual_connection=False,
     activations_checkpoint_method=None,
     activations_checkpoint_num_layers=1,
     normalization='layernorm',
     layernorm_epsilon=1e-5,
-    bias_gelu_fusion=True,
+    bias_activation_fusion=True,
     masked_softmax_fusion=True,
+    bias_dropout_add_fusion=True,
+    gradient_accumulation_fusion=False,
     persist_layer_norm=False,
     openai_gelu=False,
     onnx_safe=False,
     megatron_legacy=False,
     activations_checkpoint_granularity=None,
+    activations_checkpoint_layers_per_pipeline=None,
     sequence_parallel=False,
-    gradient_accumulation_fusion=False,
+    transformer_engine=False,
+    fp8=False,
+    fp8_e4m3=False,
+    fp8_hybrid=False,
+    fp8_margin=0,
+    fp8_interval=1,
+    fp8_amax_history_len=1,
+    fp8_amax_compute_algo='most_recent',
+    reduce_amax=True,
+    use_emha=False,
 ):
     """Build language model and return along with the key to save."""
 
@@ -110,21 +123,34 @@ def get_language_model(
         post_process=post_process,
         use_cpu_initialization=use_cpu_initialization,
         hidden_dropout=hidden_dropout,
+        attention_dropout=attention_dropout,
         precision=precision,
         fp32_residual_connection=fp32_residual_connection,
         activations_checkpoint_method=activations_checkpoint_method,
         activations_checkpoint_num_layers=activations_checkpoint_num_layers,
         normalization=normalization,
         layernorm_epsilon=layernorm_epsilon,
-        bias_gelu_fusion=bias_gelu_fusion,
+        bias_activation_fusion=bias_activation_fusion,
+        bias_dropout_add_fusion=bias_dropout_add_fusion,
         masked_softmax_fusion=masked_softmax_fusion,
+        gradient_accumulation_fusion=gradient_accumulation_fusion,
         persist_layer_norm=persist_layer_norm,
         openai_gelu=openai_gelu,
         onnx_safe=onnx_safe,
         megatron_legacy=megatron_legacy,
         activations_checkpoint_granularity=activations_checkpoint_granularity,
+        activations_checkpoint_layers_per_pipeline=activations_checkpoint_layers_per_pipeline,
         sequence_parallel=sequence_parallel,
-        gradient_accumulation_fusion=gradient_accumulation_fusion,
+        transformer_engine=transformer_engine,
+        fp8=fp8,
+        fp8_e4m3=fp8_e4m3,
+        fp8_hybrid=fp8_hybrid,
+        fp8_margin=fp8_margin,
+        fp8_interval=fp8_interval,
+        fp8_amax_history_len=fp8_amax_history_len,
+        fp8_amax_compute_algo=fp8_amax_compute_algo,
+        reduce_amax=reduce_amax,
+        use_emha=use_emha,
     )
     # key used for checkpoints.
     language_model_key = 'language_model'
@@ -156,7 +182,7 @@ class Pooler(MegatronModule):
         # gather data along sequence dimensions
         # same pooler is run on all tensor parallel nodes
         if self.sequence_parallel:
-            hidden_states = tensor_parallel.gather_from_sequence_parallel_region()
+            hidden_states = tensor_parallel.mappings.gather_from_sequence_parallel_region(hidden_states)
 
         pooled = hidden_states[sequence_index, :, :]
         pooled = self.dense(pooled)
@@ -258,10 +284,11 @@ class Embedding(MegatronModule):
         # Initialize the token-type embeddings.
         self.init_method(self.tokentype_embeddings.weight)
 
-    def forward(self, input_ids, position_ids, token_type_ids=None):
+    def forward(self, input_ids, position_ids=None, token_type_ids=None):
         # Embeddings.
         words_embeddings = self.word_embeddings(input_ids)
         if self.position_embedding_type == 'learned_absolute':
+            assert position_ids is not None
             position_embeddings = self.position_embeddings(position_ids)
             embeddings = words_embeddings + position_embeddings
         else:
@@ -385,21 +412,34 @@ class TransformerLanguageModel(MegatronModule):
         post_process=True,
         use_cpu_initialization=False,
         hidden_dropout=0.1,
+        attention_dropout=0.1,
         precision=16,
         fp32_residual_connection=False,
         activations_checkpoint_method=None,
         activations_checkpoint_num_layers=1,
         normalization='layernorm',
         layernorm_epsilon=1e-5,
-        bias_gelu_fusion=True,
+        bias_activation_fusion=True,
+        bias_dropout_add_fusion=True,
         masked_softmax_fusion=True,
+        gradient_accumulation_fusion=False,
         persist_layer_norm=False,
         openai_gelu=False,
         onnx_safe=False,
         megatron_legacy=False,
         activations_checkpoint_granularity=None,
+        activations_checkpoint_layers_per_pipeline=None,
         sequence_parallel=False,
-        gradient_accumulation_fusion=False,
+        transformer_engine=False,
+        fp8=False,
+        fp8_e4m3=False,
+        fp8_hybrid=False,
+        fp8_margin=0,
+        fp8_interval=1,
+        fp8_amax_history_len=1,
+        fp8_amax_compute_algo='most_recent',
+        reduce_amax=True,
+        use_emha=False,
     ):
         super(TransformerLanguageModel, self).__init__()
 
@@ -460,16 +500,29 @@ class TransformerLanguageModel(MegatronModule):
             normalization=normalization,
             layernorm_epsilon=layernorm_epsilon,
             hidden_dropout=hidden_dropout,
+            attention_dropout=attention_dropout,
             use_cpu_initialization=use_cpu_initialization,
-            bias_activation_fusion=bias_gelu_fusion,
             persist_layer_norm=persist_layer_norm,
             openai_gelu=openai_gelu,
             onnx_safe=onnx_safe,
+            bias_activation_fusion=bias_activation_fusion,
+            bias_dropout_add_fusion=bias_dropout_add_fusion,
             masked_softmax_fusion=masked_softmax_fusion,
+            gradient_accumulation_fusion=gradient_accumulation_fusion,
             megatron_legacy=megatron_legacy,
             sequence_parallel=sequence_parallel,
             activations_checkpoint_granularity=activations_checkpoint_granularity,
-            gradient_accumulation_fusion=gradient_accumulation_fusion,
+            activations_checkpoint_layers_per_pipeline=activations_checkpoint_layers_per_pipeline,
+            transformer_engine=transformer_engine,
+            fp8=fp8,
+            fp8_e4m3=fp8_e4m3,
+            fp8_hybrid=fp8_hybrid,
+            fp8_margin=fp8_margin,
+            fp8_interval=fp8_interval,
+            fp8_amax_history_len=fp8_amax_history_len,
+            fp8_amax_compute_algo=fp8_amax_compute_algo,
+            reduce_amax=reduce_amax,
+            use_emha=use_emha,
         )
         self._encoder_key = 'encoder'
 
@@ -495,16 +548,20 @@ class TransformerLanguageModel(MegatronModule):
                 normalization=normalization,
                 layernorm_epsilon=layernorm_epsilon,
                 hidden_dropout=hidden_dropout,
+                attention_dropout=attention_dropout,
                 use_cpu_initialization=use_cpu_initialization,
-                bias_activation_fusion=bias_gelu_fusion,
+                bias_activation_fusion=bias_activation_fusion,
+                bias_dropout_add_fusion=bias_dropout_add_fusion,
+                masked_softmax_fusion=masked_softmax_fusion,
+                gradient_accumulation_fusion=gradient_accumulation_fusion,
                 persist_layer_norm=persist_layer_norm,
                 openai_gelu=openai_gelu,
                 onnx_safe=onnx_safe,
-                masked_softmax_fusion=masked_softmax_fusion,
                 megatron_legacy=megatron_legacy,
                 sequence_parallel=sequence_parallel,
                 activations_checkpoint_granularity=activations_checkpoint_granularity,
-                gradient_accumulation_fusion=gradient_accumulation_fusion,
+                activations_checkpoint_layers_per_pipeline=activations_checkpoint_layers_per_pipeline,
+                transformer_engine=transformer_engine,
             )
             self._decoder_key = 'decoder'
 
@@ -541,6 +598,7 @@ class TransformerLanguageModel(MegatronModule):
         encoder_input=None,
         set_inference_key_value_memory=False,
         inference_max_sequence_len=None,
+        checkpoint_activations_all_layers=None,
     ):
         # Embeddings.
         if self.pre_process and encoder_input is None:
@@ -561,6 +619,7 @@ class TransformerLanguageModel(MegatronModule):
                 get_key_value=get_key_value,
                 set_inference_key_value_memory=set_inference_key_value_memory,
                 inference_max_sequence_len=inference_max_sequence_len,
+                checkpoint_activations_all_layers=checkpoint_activations_all_layers,
             )
         else:
             encoder_output = enc_hidden_states.to(encoder_input.dtype)
@@ -590,6 +649,7 @@ class TransformerLanguageModel(MegatronModule):
             enc_dec_attn_mask=enc_dec_attn_mask,
             set_inference_key_value_memory=set_inference_key_value_memory,
             inference_max_sequence_len=inference_max_sequence_len,
+            checkpoint_activations_all_layers=checkpoint_activations_all_layers,
         )
 
         if self.add_pooler and self.post_process:
