@@ -56,6 +56,7 @@ def get_encoder_model(
     use_cpu_initialization=False,
     hidden_dropout=0.1,
     attention_dropout=0.1,
+    ffn_dropout=0.0,
     precision=16,
     fp32_residual_connection=False,
     activations_checkpoint_method=None,
@@ -79,8 +80,15 @@ def get_encoder_model(
     chunk_size=64,
     num_self_attention_per_cross_attention=1,
     layer_number_offset=0,  # this is use only for attention norm_factor scaling
+    megatron_legacy=False,
+    normalize_attention_scores=True,
     sequence_parallel=False,
     gradient_accumulation_fusion=False,
+    num_moe_experts=1,
+    moe_frequency=1,
+    moe_dropout=0.0,
+    turn_off_rop=False,  # turn off the RoP positional embedding
+    version=1,  # model version
 ):
     """Build language model and return along with the key to save."""
 
@@ -113,10 +121,12 @@ def get_encoder_model(
             use_cpu_initialization=use_cpu_initialization,
             hidden_dropout=hidden_dropout,
             attention_dropout=attention_dropout,
+            ffn_dropout=ffn_dropout,
             precision=precision,
             fp32_residual_connection=fp32_residual_connection,
             activations_checkpoint_method=activations_checkpoint_method,
             activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+            activations_checkpoint_granularity=activations_checkpoint_granularity,
             layernorm_epsilon=layernorm_epsilon,
             bias_activation_fusion=bias_activation_fusion,
             bias_dropout_add_fusion=bias_dropout_add_fusion,
@@ -130,6 +140,11 @@ def get_encoder_model(
             transformer_block_type=transformer_block_type,
             headscale=headscale,
             parent_model_type=parent_model_type,
+            megatron_legacy=megatron_legacy,
+            normalize_attention_scores=normalize_attention_scores,
+            num_moe_experts=num_moe_experts,
+            moe_frequency=moe_frequency,
+            moe_dropout=moe_dropout,
         )
     elif arch == "retro":
         encoder = MegatronRetrievalTransformerEncoderModule(
@@ -166,8 +181,12 @@ def get_encoder_model(
             parent_model_type=parent_model_type,
             chunk_size=chunk_size,
             layer_number_offset=layer_number_offset,
+            megatron_legacy=megatron_legacy,
+            normalize_attention_scores=normalize_attention_scores,
             sequence_parallel=sequence_parallel,
             gradient_accumulation_fusion=gradient_accumulation_fusion,
+            turn_off_rop=turn_off_rop,
+            version=version,
         )
     elif arch == "perceiver":
         encoder = MegatronPerceiverEncoderModule(
@@ -185,10 +204,12 @@ def get_encoder_model(
             use_cpu_initialization=use_cpu_initialization,
             hidden_dropout=hidden_dropout,
             attention_dropout=attention_dropout,
+            ffn_dropout=ffn_dropout,
             precision=precision,
             fp32_residual_connection=fp32_residual_connection,
             activations_checkpoint_method=activations_checkpoint_method,
             activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+            activations_checkpoint_granularity=activations_checkpoint_granularity,
             layernorm_epsilon=layernorm_epsilon,
             bias_activation_fusion=bias_activation_fusion,
             bias_dropout_add_fusion=bias_dropout_add_fusion,
@@ -204,6 +225,8 @@ def get_encoder_model(
             parent_model_type=parent_model_type,
             hidden_steps=hidden_steps,
             num_self_attention_per_cross_attention=num_self_attention_per_cross_attention,
+            megatron_legacy=megatron_legacy,
+            normalize_attention_scores=normalize_attention_scores,
         )
     else:
         raise ValueError(f"Unknown encoder arch = {arch}. Available encoder arch = {AVAILABLE_ENCODERS}")
