@@ -23,7 +23,6 @@ from nemo.collections.nlp.modules.common.megatron.module import MegatronModule
 from nemo.collections.nlp.modules.common.megatron.mup.layer import MuReadout
 from nemo.collections.nlp.modules.common.megatron.utils import (
     ApexGuardDefaults,
-    build_position_ids,
     init_method_normal,
     scaled_init_method_normal,
 )
@@ -90,10 +89,17 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
         dec_cross_attention=[3, 5],  # layer numbers for chunked cross attention
         add_position_embedding=False,
         tokenizer=None,  # tokenizer
+        normalize_attention_scores=True,
         activations_checkpoint_granularity=None,
+        megatron_lm_compatible=False,
+        version=1,
     ):
         super(MegatronRetrievalTokenLevelEncoderDecoderModule, self).__init__()
-
+        if megatron_lm_compatible:
+            assert (
+                apply_query_key_layer_scaling
+            ), "megatron lm compatible model has to set apply_query_key_layer_scaling"
+            assert add_position_embedding, "megatron lm compatible model has to set add_position_embedding"
         self.parallel_output = parallel_output
         self.pre_process = pre_process
         self.post_process = post_process
@@ -106,6 +112,7 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
         self.eod_id = tokenizer.eos_id
         self.transformer_block_type = transformer_block_type
         self.num_chunked_cross_attention = len(dec_cross_attention)
+        self.megatron_lm_compatible = megatron_lm_compatible
 
         if kv_channels is None:
             assert (
@@ -153,7 +160,9 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
                 init_method=encoder_init,
                 scaled_init_method=encoder_scaled_init,
                 pre_process=pre_process,
-                post_process=post_process,
+                post_process=False
+                if megatron_lm_compatible
+                else post_process,  # megatron lm model has no final layer_norm
                 init_method_std=init_method_std,
                 use_cpu_initialization=use_cpu_initialization,
                 hidden_dropout=hidden_dropout,
@@ -180,6 +189,9 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
                 layer_type=enc_layer_types,
                 chunk_size=chunk_size,
                 layer_number_offset=0,
+                normalize_attention_scores=normalize_attention_scores,
+                turn_off_rop=megatron_lm_compatible,
+                version=version,
             )
             self._encoder_key = "encoder"
 
@@ -241,6 +253,9 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
                 layer_type=pre_decoder_layer_types,
                 chunk_size=chunk_size,
                 layer_number_offset=0,
+                normalize_attention_scores=normalize_attention_scores,
+                turn_off_rop=megatron_lm_compatible,
+                version=version,
             )
 
             # it is where the chunked cross attention happens
@@ -282,6 +297,9 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
                 layer_type=post_decoder_layer_types,
                 chunk_size=chunk_size,
                 layer_number_offset=pre_decoder_num_layers + 1,
+                normalize_attention_scores=normalize_attention_scores,
+                turn_off_rop=megatron_lm_compatible,
+                version=version,
             )
             self._pre_decoder_key = "pre_decoder"
             self._post_decoder_key = "post_decoder"
@@ -294,6 +312,16 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
             self.tokens_head = MuReadout(self.word_embeddings_weight().size(0), parallel_output)
             self._tokens_head_key = 'tokens_head'
 
+    def set_input_tensor(self, input_tensor):
+        """Set input tensor to be used instead of forward()'s input.
+
+        When doing pipeline parallelism the input from the previous
+        stage comes from communication, not from the input, so the
+        model's forward_step_func won't have it. This function is thus
+        used by internal code to bypass the input provided by the
+        forward_step_func"""
+        self.input_tensor = input_tensor
+
     def forward(
         self,
         input_ids,
@@ -306,20 +334,22 @@ class MegatronRetrievalTokenLevelEncoderDecoderModule(MegatronModule):
         set_inference_key_value_memory=False,
         inference_max_sequence_len=None,
         neighbors=None,
+        position_ids=None,
     ):
         """
         Return value is per token / per dimension (i.e., non collapsed loss value)
         """
         eod_positions = None
         retrieved_emb = None
-        if input_ids is not None and self.eod_id is not None:
+        if input_ids is not None and self.eod_id is not None and not self.megatron_lm_compatible:
+            # do not reset attention for megatron lm compatible model
             eod_positions = torch.where(input_ids == self.eod_id)
 
         if input_emb is None:
             if self.pre_process and self.add_encoder:
                 # encoder embeddings
                 if self.add_abs_position_embedding:
-                    input_position_ids = build_position_ids(input_ids)
+                    input_position_ids = position_ids
                 else:
                     input_position_ids = None
                 input_emb = self.encoder_embedding(input_ids, input_position_ids, token_type_ids=token_type_ids)
