@@ -13,13 +13,13 @@
 # limitations under the License.
 from abc import ABC, abstractmethod
 from contextlib import ExitStack, contextmanager
+from typing import List
 
 import torch
 
 from nemo.collections.tts.helpers.helpers import OperationMode
-from nemo.collections.tts.models import *  # Avoid circular imports
 from nemo.core.classes import ModelPT
-from nemo.core.classes.common import typecheck
+from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.core.neural_types.elements import AudioSignal
 from nemo.core.neural_types.neural_type import NeuralType
 
@@ -65,15 +65,18 @@ class SpectrogramGenerator(ModelPT, ABC):
 
 
 class Vocoder(ModelPT, ABC):
-    """ Base class for all TTS models that generate audio conditioned a on spectrogram """
+    """
+    A base class for models that convert spectrograms to audios. Note that this class takes as input either linear
+    or mel spectrograms.
+    """
 
     @abstractmethod
     def convert_spectrogram_to_audio(self, spec: 'torch.tensor', **kwargs) -> 'torch.tensor':
         """
-        Accepts a batch of spectrograms and returns a batch of audio
+        Accepts a batch of spectrograms and returns a batch of audio.
 
         Args:
-            spec: A torch tensor representing the spectrograms to be vocoded
+            spec:  ['B', 'n_freqs', 'T'], A torch tensor representing the spectrograms to be vocoded.
 
         Returns:
             audio
@@ -143,9 +146,15 @@ class GlowVocoder(Vocoder):
                 ) from e
 
             def yet_another_patch(audio, n_fft, hop_length, win_length, window):
-                spec = torch.stft(audio, n_fft=n_fft, hop_length=hop_length, win_length=win_length, window=window)
-                if spec.dtype in [torch.cfloat, torch.cdouble]:
-                    spec = torch.view_as_real(spec)
+                spec = torch.stft(
+                    audio,
+                    n_fft=n_fft,
+                    hop_length=hop_length,
+                    win_length=win_length,
+                    window=window,
+                    return_complex=True,
+                )
+                spec = torch.view_as_real(spec)
                 return torch.sqrt(spec.pow(2).sum(-1)), torch.atan2(spec[..., -1], spec[..., 0])
 
             self.stft = lambda x: yet_another_patch(
@@ -194,39 +203,6 @@ class GlowVocoder(Vocoder):
         return audio_denoised
 
 
-class LinVocoder(ModelPT, ABC):
-    """
-    A base class for models that convert from the linear (magnitude) spectrogram to audio. Note: The `Vocoder` class
-    differs from this class as the `Vocoder` class takes as input mel spectrograms.
-    """
-
-    @abstractmethod
-    def convert_linear_spectrogram_to_audio(self, spec: 'torch.tensor', **kwargs) -> 'torch.tensor':
-        """
-        Accepts a batch of linear spectrograms and returns a batch of audio
-
-        Args:
-            spec: A torch tensor representing the linear spectrograms to be vocoded ['B', 'n_freqs', 'T']
-
-        Returns:
-            audio
-        """
-
-    @classmethod
-    def list_available_models(cls) -> 'List[PretrainedModelInfo]':
-        """
-        This method returns a list of pre-trained model which can be instantiated directly from NVIDIA's NGC cloud.
-        Returns:
-            List of available pre-trained models.
-        """
-        list_of_models = []
-        for subclass in cls.__subclasses__():
-            subclass_models = subclass.list_available_models()
-            if subclass_models is not None and len(subclass_models) > 0:
-                list_of_models.extend(subclass_models)
-        return list_of_models
-
-
 class MelToSpec(ModelPT, ABC):
     """
     A base class for models that convert mel spectrograms to linear (magnitude) spectrograms
@@ -265,7 +241,7 @@ class TextToWaveform(ModelPT, ABC):
     @abstractmethod
     def parse(self, str_input: str, **kwargs) -> 'torch.tensor':
         """
-        A helper function that accepts raw python strings and turns them into a tensor. The tensor should have 2
+       A helper function that accepts a raw python string and turns it into a tensor. The tensor should have 2
         dimensions. The first is the batch, which should be of size 1. The second should represent time. The tensor
         should represent either tokenized or embedded text, depending on the model.
         """
@@ -274,10 +250,8 @@ class TextToWaveform(ModelPT, ABC):
     def convert_text_to_waveform(self, *, tokens: 'torch.tensor', **kwargs) -> 'List[torch.tensor]':
         """
         Accepts a batch of text and returns a list containing a batch of audio
-
         Args:
             tokens: A torch tensor representing the text to be converted to speech
-
         Returns:
             audio: A list of length batch_size containing torch tensors representing the waveform output
         """
