@@ -286,7 +286,8 @@ class HifiGanModel(Vocoder, Exportable):
 
     def _bias_denoise(self, audio, mel):
         def stft(x):
-            comp = torch.stft(x.squeeze(1), n_fft=1024, hop_length=256, win_length=1024)
+            comp = torch.stft(x.squeeze(1), n_fft=1024, hop_length=256, win_length=1024, return_complex=True)
+            comp = torch.view_as_real(comp)
             real, imag = comp[..., 0], comp[..., 1]
             mags = torch.sqrt(real ** 2 + imag ** 2)
             phase = torch.atan2(imag, real)
@@ -294,7 +295,7 @@ class HifiGanModel(Vocoder, Exportable):
 
         def istft(mags, phase):
             comp = torch.stack([mags * torch.cos(phase), mags * torch.sin(phase)], dim=-1)
-            x = torch.istft(comp, n_fft=1024, hop_length=256, win_length=1024)
+            x = torch.istft(torch.view_as_complex(comp), n_fft=1024, hop_length=256, win_length=1024)
             return x
 
         # Create bias tensor
@@ -402,6 +403,28 @@ class HifiGanModel(Vocoder, Exportable):
         )
         list_of_models.append(model)
 
+        # Spanish, multi-speaker, 44100 Hz, Latin American Spanish OpenSLR
+        model = PretrainedModelInfo(
+            pretrained_model_name="tts_es_hifigan_ft_fastpitch_multispeaker",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_es_multispeaker_fastpitchhifigan/versions/1.15.0/files/tts_es_hifigan_ft_fastpitch_multispeaker.nemo",
+            description="This model is trained on the audio from 6 crowdsourced Latin American Spanish OpenSLR "
+            "datasets and finetuned on the mel-spectrograms generated from the FastPitch checkpoint "
+            "`tts_es_fastpitch_multispeaker`. This model has been tested on generating male and female "
+            "Spanish voices with Latin American accents.",
+            class_=cls,
+        )
+        list_of_models.append(model)
+
+        # zh, single female speaker, 22050 Hz, SFSpeech Chinese/English Bilingual Dataset.
+        model = PretrainedModelInfo(
+            pretrained_model_name="tts_zh_hifigan_sfspeech",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_zh_fastpitch_hifigan_sfspeech/versions/1.14.0/files/tts_zh_hifigan_sfspeech.nemo",
+            description="This model is finetuned from the HiFiGAN pretrained checkpoint `tts_en_hifitts_hifigan_ft_fastpitch` "
+            "by the mel-spectrograms generated from the FastPitch checkpoint `tts_zh_fastpitch_sfspeech`. This model "
+            "has been tested on generating female Mandarin Chinese voices.",
+            class_=cls,
+        )
+        list_of_models.append(model)
         return list_of_models
 
     def load_state_dict(self, state_dict, strict=True):
