@@ -11,24 +11,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-# ##########################################################################
-
+import contextlib
+import random
 
 import torch
 from hydra.utils import instantiate
 from omegaconf import DictConfig, OmegaConf
 from pytorch_lightning import Trainer
-from pytorch_lightning.loggers import LoggerCollection, TensorBoardLogger
+from pytorch_lightning.loggers import TensorBoardLogger
 
 from nemo.collections.common.tokenizers.text_to_speech.tts_tokenizers import BaseTokenizer
-from nemo.collections.tts.helpers.helpers import plot_alignment_to_numpy
+from nemo.collections.tts.helpers.helpers import plot_alignment_to_numpy, regulate_len
 from nemo.collections.tts.losses.radttsloss import AttentionBinarizationLoss, RADTTSLoss
 from nemo.collections.tts.models.base import SpectrogramGenerator
 from nemo.core.classes import Exportable
 from nemo.core.classes.common import typecheck
-from nemo.core.neural_types.elements import Index, MelSpectrogramType, TokenIndex
+from nemo.core.neural_types.elements import (
+    Index,
+    MelSpectrogramType,
+    RegressionValuesType,
+    TokenDurationType,
+    TokenIndex,
+)
 from nemo.core.neural_types.neural_type import NeuralType
+from nemo.core.optim.radam import RAdam
 from nemo.utils import logging
 from nemo.utils.decorators import experimental
 
@@ -38,8 +44,13 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
         if isinstance(cfg, dict):
             cfg = OmegaConf.create(cfg)
+        self.normalizer = None
+        self.text_normalizer_call = None
+        self.text_normalizer_call_kwargs = {}
+        self._setup_normalizer(cfg)
 
-        self._setup_tokenizer(cfg.validation_ds.dataset)
+        self.tokenizer = None
+        self._setup_tokenizer(cfg)
 
         assert self.tokenizer is not None
 
@@ -70,11 +81,7 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
         self._tb_logger = None
         self.cfg = cfg
         self.log_train_images = False
-
-        self.normalizer = None
-        self.text_normalizer_call = None
-        self.text_normalizer_call_kwargs = {}
-        self._setup_normalizer(cfg)
+        # print("intial self normalizer", self.normalizer)
 
     def batch_dict(self, batch_data):
         if len(batch_data) < 14:
@@ -154,7 +161,7 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
         loss_outputs['binarization_loss'] = (binarization_loss, 1.0)
 
         for k, (v, w) in loss_outputs.items():
-            self.log("train/" + k, loss_outputs[k][0])
+            self.log("train/" + k, loss_outputs[k][0], on_step=True)
 
         return {'loss': loss}
 
@@ -224,7 +231,7 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
 
         for k, v in loss_outputs.items():
             if k != "binarization_loss":
-                self.log("val/" + k, loss_outputs[k][0])
+                self.log("val/" + k, loss_outputs[k][0], sync_dist=True, on_epoch=True)
 
         attn = outputs[0]["attn"]
         attn_soft = outputs[0]["attn_soft"]
@@ -258,24 +265,26 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
                 self.model.parameters(), lr=self.optim.lr, weight_decay=self.optim.weight_decay
             )
         elif self.optim.name == 'RAdam':  # False for inference riva
-            optimizer = torch.optim.RAdam(
-                self.model.parameters(), lr=self.optim.lr, weight_decay=self.optim.weight_decay
-            )
+            optimizer = RAdam(self.model.parameters(), lr=self.optim.lr, weight_decay=self.optim.weight_decay)
         else:
             logging.info("Unrecognized optimizer %s! Please choose the right optimizer" % (self.optim.name))
             exit(1)
 
         return optimizer
 
-    @staticmethod
-    def _loader(cfg):
+    def _loader(self, cfg):
         try:
             _ = cfg.dataset.manifest_filepath
         except omegaconf.errors.MissingMandatoryValue:
             logging.warning("manifest_filepath was skipped. No dataset for this model.")
             return None
-
-        dataset = instantiate(cfg.dataset)
+        # print("inside loader self normalizer", self.normalizer)
+        dataset = instantiate(
+            cfg.dataset,
+            text_normalizer=self.normalizer,
+            text_normalizer_call_kwargs=self.text_normalizer_call_kwargs,
+            text_tokenizer=self.tokenizer,
+        )
         return torch.utils.data.DataLoader(  # noqa
             dataset=dataset, collate_fn=dataset.collate_fn, **cfg.dataloader_params,
         )
@@ -364,7 +373,17 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
             logging.warning("parse() is meant to be called in eval mode.")
         if normalize and self.text_normalizer_call is not None:
             text = self.text_normalizer_call(text, **self.text_normalizer_call_kwargs)
-        return torch.tensor(self.tokenizer(text)).long().unsqueeze(0).cuda().to(self.device)
+
+        eval_phon_mode = contextlib.nullcontext()
+        if hasattr(self.tokenizer, "set_phone_prob"):
+            eval_phon_mode = self.tokenizer.set_phone_prob(prob=1)
+            print("changed to one")
+
+        with eval_phon_mode:
+            tokens = self.tokenizer.encode(text)
+        print("text to token phone_prob")
+
+        return torch.tensor(tokens).long().unsqueeze(0).cuda().to(self.device)
 
     @property
     def tb_logger(self):
@@ -372,24 +391,129 @@ class RadTTSModel(SpectrogramGenerator, Exportable):
             if self.logger is None and self.logger.experiment is None:
                 return None
             tb_logger = self.logger.experiment
-            if isinstance(self.logger, LoggerCollection):
-                for logger in self.logger:
-                    if isinstance(logger, TensorBoardLogger):
-                        tb_logger = logger.experiment
-                        break
+            for logger in self.trainer.loggers:
+                if isinstance(logger, TensorBoardLogger):
+                    tb_logger = logger.experiment
+                    break
             self._tb_logger = tb_logger
         return self._tb_logger
 
+    def load_state_dict(self, state_dict, strict=True):
+        # Override load_state_dict to be backward-compatible with old checkpoints
+        new_state_dict = {}
+        for k, v in state_dict.items():
+            k = k.replace("projection_fn.weight", "projection_fn.conv.weight")
+            k = k.replace("projection_fn.bias", "projection_fn.conv.bias")
+            new_state_dict[k] = v
+        super().load_state_dict(new_state_dict, strict=strict)
+
+    # Methods for model exportability
     @property
-    def input_module(self):
-        return self.model
+    def input_types(self):
+        return self._input_types
 
     @property
-    def output_module(self):
-        return self.model
+    def output_types(self):
+        return self._output_types
 
-    def forward_for_export(self, text, lens, speaker_id, speaker_id_text, speaker_id_attributes):
-        return self.model.forward_for_export(text, lens, speaker_id, speaker_id_text, speaker_id_attributes)
+    def _prepare_for_export(self, **kwargs):
+        self.model.remove_norms()
+        super()._prepare_for_export(**kwargs)
 
-    def get_export_subnet(self, subnet=None):
-        return self.model.get_export_subnet(subnet)
+        # Define input_types and output_types as required by export()
+        self._input_types = {
+            "text": NeuralType(('B', 'T'), TokenIndex()),
+            "lens": NeuralType(('B')),
+            # "batch_lengths": NeuralType(('B'), LengthsType(), optional=True),
+            "speaker_id": NeuralType(('B'), Index()),
+            "speaker_id_text": NeuralType(('B'), Index()),
+            "speaker_id_attributes": NeuralType(('B'), Index()),
+            "pitch": NeuralType(('B', 'T'), RegressionValuesType()),
+            "pace": NeuralType(('B', 'T')),
+            "volume": NeuralType(('B', 'T'), optional=True),
+        }
+        self._output_types = {
+            "spect": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType()),
+            "num_frames": NeuralType(('B'), TokenDurationType()),
+            "durs_predicted": NeuralType(('B', 'T_text'), TokenDurationType()),
+            "volume_aligned": NeuralType(('B', 'T_spec'), RegressionValuesType()),
+        }
+
+    def input_example(self, max_batch=1, max_dim=400):
+        par = next(self.parameters())
+        sz = (max_batch, max_dim)
+        # sz = (max_batch * max_dim,)
+        inp = torch.randint(0, 94, sz, device=par.device, dtype=torch.int64)
+        speaker = torch.randint(0, 1, (max_batch,), device=par.device, dtype=torch.int64)
+        pitch = torch.randn(sz, device=par.device, dtype=torch.float32) * 0.5
+        pace = torch.clamp(torch.randn(sz, device=par.device, dtype=torch.float32) * 0.1 + 1, min=0.01)
+        volume = torch.clamp(torch.randn(sz, device=par.device, dtype=torch.float32) * 0.1 + 1, min=0.01)
+        # batch_lengths = torch.zeros((max_batch + 1), device=par.device, dtype=torch.int32)
+        # left_over_size = sz[0]
+        # batch_lengths[0] = 0
+        # for i in range(1, max_batch):
+        #     length = torch.randint(1, left_over_size - (max_batch - i), (1,), device=par.device)
+        #     batch_lengths[i] = length + batch_lengths[i - 1]
+        #     left_over_size -= length.detach().cpu().numpy()[0]
+        # batch_lengths[-1] = left_over_size + batch_lengths[-2]
+        # sum = 0
+        # index = 1
+        # while index < len(batch_lengths):
+        #     sum += batch_lengths[index] - batch_lengths[index - 1]
+        #     index += 1
+        # assert sum == sz[0], f"sum: {sum}, sz: {sz[0]}, lengths:{batch_lengths}"
+
+        pad_id = self.tokenizer.pad
+        inp[inp == pad_id] = pad_id - 1 if pad_id > 0 else pad_id + 1
+
+        lens = []
+        for i, _ in enumerate(inp):
+            len_i = random.randint(3, max_dim)
+            lens.append(len_i)
+            inp[i, len_i:] = pad_id
+        lens = torch.tensor(lens, device=par.device, dtype=torch.int)
+
+        inputs = {
+            'text': inp,
+            'lens': lens,
+            # 'batch_lengths': batch_lengths,
+            'speaker_id': speaker,
+            'speaker_id_text': speaker,
+            'speaker_id_attributes': speaker,
+            'pitch': pitch,
+            'pace': pace,
+            'volume': volume,
+        }
+        return (inputs,)
+
+    def forward_for_export(
+        self, text, lens, speaker_id, speaker_id_text, speaker_id_attributes, pitch, pace, volume,
+    ):
+        (mel, n_frames, dur, _, _) = self.model.infer(
+            speaker_id,
+            text,
+            speaker_id_text=speaker_id_text,
+            speaker_id_attributes=speaker_id_attributes,
+            sigma=0.0,
+            sigma_txt=0.0,
+            sigma_f0=1.0,
+            sigma_energy=1.0,
+            f0_mean=0.0,
+            f0_std=0.0,
+            in_lens=lens,
+            pitch_shift=pitch,
+            pace=pace,
+        ).values()
+        # Need to reshape as in infer patch
+        durs_predicted = dur.float()
+        truncated_length = torch.max(lens)
+        volume_extended, _ = regulate_len(
+            durs_predicted,
+            volume[:, :truncated_length].unsqueeze(-1),
+            pace[:, :truncated_length],
+            replicate_to_nearest_multiple=True,
+            group_size=self.model.n_group_size,
+            in_lens=lens,
+        )
+        volume_extended = volume_extended.squeeze(-1).float()
+        return mel.float(), n_frames, dur.float(), volume_extended
