@@ -61,7 +61,7 @@ from nemo.core.neural_types.elements import (
 from nemo.core.neural_types.neural_type import NeuralType
 
 
-def average_pitch(pitch, durs):
+def average_features(pitch, durs):
     durs_cums_ends = torch.cumsum(durs, dim=1).long()
     durs_cums_starts = torch.nn.functional.pad(durs_cums_ends[:, :-1], (1, 0))
     pitch_nonzero_cums = torch.nn.functional.pad(torch.cumsum(pitch != 0.0, dim=2), (1, 0))
@@ -135,12 +135,17 @@ class FastPitchModule(NeuralModule):
         decoder_module: NeuralModule,
         duration_predictor: NeuralModule,
         pitch_predictor: NeuralModule,
+        energy_predictor: NeuralModule,
         aligner: NeuralModule,
         n_speakers: int,
         symbols_embedding_dim: int,
         pitch_embedding_kernel_size: int,
+        energy_embedding_kernel_size: int,
         n_mel_channels: int = 80,
         max_token_duration: int = 75,
+        speaker_emb_condition_prosody: bool = False,
+        speaker_emb_condition_decoder: bool = False,
+        speaker_emb_condition_aligner: bool = False,
     ):
         super().__init__()
 
@@ -148,10 +153,14 @@ class FastPitchModule(NeuralModule):
         self.decoder = decoder_module
         self.duration_predictor = duration_predictor
         self.pitch_predictor = pitch_predictor
+        self.energy_predictor = energy_predictor
         self.aligner = aligner
         self.learn_alignment = aligner is not None
         self.use_duration_predictor = True
         self.binarize = False
+        self.speaker_emb_condition_prosody = speaker_emb_condition_prosody
+        self.speaker_emb_condition_decoder = speaker_emb_condition_decoder
+        self.speaker_emb_condition_aligner = speaker_emb_condition_aligner
 
         if n_speakers > 1:
             self.speaker_emb = torch.nn.Embedding(n_speakers, symbols_embedding_dim)
@@ -168,6 +177,14 @@ class FastPitchModule(NeuralModule):
             padding=int((pitch_embedding_kernel_size - 1) / 2),
         )
 
+        if self.energy_predictor is not None:
+            self.energy_emb = torch.nn.Conv1d(
+                1,
+                symbols_embedding_dim,
+                kernel_size=energy_embedding_kernel_size,
+                padding=int((energy_embedding_kernel_size - 1) / 2),
+            )
+
         # Store values precomputed from training data for convenience
         self.register_buffer('pitch_mean', torch.zeros(1))
         self.register_buffer('pitch_std', torch.zeros(1))
@@ -180,6 +197,7 @@ class FastPitchModule(NeuralModule):
             "text": NeuralType(('B', 'T_text'), TokenIndex()),
             "durs": NeuralType(('B', 'T_text'), TokenDurationType()),
             "pitch": NeuralType(('B', 'T_audio'), RegressionValuesType()),
+            "energy": NeuralType(('B', 'T_audio'), RegressionValuesType(), optional=True),
             "speaker": NeuralType(('B'), Index(), optional=True),
             "pace": NeuralType(optional=True),
             "spec": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType(), optional=True),
@@ -201,6 +219,8 @@ class FastPitchModule(NeuralModule):
             "attn_hard": NeuralType(('B', 'S', 'T_spec', 'T_text'), ProbsType()),
             "attn_hard_dur": NeuralType(('B', 'T_text'), TokenDurationType()),
             "pitch": NeuralType(('B', 'T_audio'), RegressionValuesType()),
+            "energy_pred": NeuralType(('B', 'T_text'), RegressionValuesType()),
+            "energy_tgt": NeuralType(('B', 'T_audio'), RegressionValuesType()),
         }
 
     @typecheck()
@@ -210,6 +230,7 @@ class FastPitchModule(NeuralModule):
         text,
         durs=None,
         pitch=None,
+        energy=None,
         speaker=None,
         pace=1.0,
         spec=None,
@@ -230,29 +251,62 @@ class FastPitchModule(NeuralModule):
 
         # Input FFT
         enc_out, enc_mask = self.encoder(input=text, conditioning=spk_emb)
-
-        log_durs_predicted = self.duration_predictor(enc_out, enc_mask)
+        if self.speaker_emb_condition_prosody:
+            prosody_input = enc_out + spk_emb
+        else:
+            prosody_input = enc_out
+        log_durs_predicted = self.duration_predictor(prosody_input, enc_mask)
         durs_predicted = torch.clamp(torch.exp(log_durs_predicted) - 1, 0, self.max_token_duration)
 
         attn_soft, attn_hard, attn_hard_dur, attn_logprob = None, None, None, None
         if self.learn_alignment and spec is not None:
             text_emb = self.encoder.word_emb(text)
-            attn_soft, attn_logprob = self.aligner(spec, text_emb.permute(0, 2, 1), enc_mask == 0, attn_prior)
+            if self.speaker_emb_condition_aligner and not isinstance(spk_emb, int):
+                attn_soft, attn_logprob = self.aligner(
+                    spec, text_emb.permute(0, 2, 1), enc_mask == 0, attn_prior, conditioning=spk_emb
+                )
+            else:
+                attn_soft, attn_logprob = self.aligner(spec, text_emb.permute(0, 2, 1), enc_mask == 0, attn_prior)
             attn_hard = binarize_attention_parallel(attn_soft, input_lens, mel_lens)
             attn_hard_dur = attn_hard.sum(2)[:, 0, :]
 
         # Predict pitch
-        pitch_predicted = self.pitch_predictor(enc_out, enc_mask)
+        pitch_predicted = self.pitch_predictor(prosody_input, enc_mask)
         if pitch is not None:
             if self.learn_alignment and pitch.shape[-1] != pitch_predicted.shape[-1]:
                 # Pitch during training is per spectrogram frame, but during inference, it should be per character
-                pitch = average_pitch(pitch.unsqueeze(1), attn_hard_dur).squeeze(1)
+                pitch = average_features(pitch.unsqueeze(1), attn_hard_dur).squeeze(1)
+            elif not self.learn_alignment:
+                # If alignment is not learnt attn_hard_dur is None, hence durs_predicted
+                pitch = average_features(pitch.unsqueeze(1), durs_predicted).squeeze(1)
             pitch_emb = self.pitch_emb(pitch.unsqueeze(1))
         else:
             pitch_emb = self.pitch_emb(pitch_predicted.unsqueeze(1))
 
         enc_out = enc_out + pitch_emb.transpose(1, 2)
 
+        # Predict energy
+        if self.energy_predictor is not None:
+            energy_pred = self.energy_predictor(enc_out, enc_mask).squeeze(-1)
+
+            if energy is not None:
+                # Average energy over characters
+                if self.learn_alignment:
+                    energy_tgt = average_features(energy.unsqueeze(1), attn_hard_dur)
+                else:
+                    energy_tgt = average_features(energy.unsqueeze(1), durs_predicted)
+                energy_tgt = torch.log(1.0 + energy_tgt)
+                energy_emb = self.energy_emb(energy_tgt)
+                energy_tgt = energy_tgt.squeeze(1)
+            else:
+                energy_emb = self.energy_emb(energy_pred.unsqueeze(1))
+                energy_tgt = None
+
+            enc_out = enc_out + energy_emb.transpose(1, 2)
+        else:
+            energy_pred = None
+            energy_tgt = None
+
         if self.learn_alignment and spec is not None:
             len_regulated, dec_lens = regulate_len(attn_hard_dur, enc_out, pace)
         elif spec is None and durs is not None:
@@ -262,7 +316,10 @@ class FastPitchModule(NeuralModule):
             len_regulated, dec_lens = regulate_len(durs_predicted, enc_out, pace)
 
         # Output FFT
-        dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
+        if self.speaker_emb_condition_decoder:
+            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens, conditioning=spk_emb)
+        else:
+            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
         spect = self.proj(dec_out).transpose(1, 2)
         return (
             spect,
@@ -275,9 +332,11 @@ class FastPitchModule(NeuralModule):
             attn_hard,
             attn_hard_dur,
             pitch,
+            energy_pred,
+            energy_tgt,
         )
 
-    def infer(self, *, text, pitch=None, speaker=None, pace=1.0, volume=None):
+    def infer(self, *, text, pitch=None, speaker=None, energy=None, pace=1.0, volume=None):
         # Calculate speaker embedding
         if self.speaker_emb is None or speaker is None:
             spk_emb = 0
@@ -286,14 +345,26 @@ class FastPitchModule(NeuralModule):
 
         # Input FFT
         enc_out, enc_mask = self.encoder(input=text, conditioning=spk_emb)
+        if self.speaker_emb_condition_prosody:
+            prosody_input = enc_out + spk_emb
+        else:
+            prosody_input = enc_out
 
         # Predict duration and pitch
-        log_durs_predicted = self.duration_predictor(enc_out, enc_mask)
+        log_durs_predicted = self.duration_predictor(prosody_input, enc_mask)
         durs_predicted = torch.clamp(
             torch.exp(log_durs_predicted) - 1.0, self.min_token_duration, self.max_token_duration
         )
-        pitch_predicted = self.pitch_predictor(enc_out, enc_mask) + pitch
+        pitch_predicted = self.pitch_predictor(prosody_input, enc_mask) + pitch
         pitch_emb = self.pitch_emb(pitch_predicted.unsqueeze(1))
+        if self.energy_predictor is not None:
+            if energy is not None:
+                assert energy.shape[-1] == text.shape[-1], f"energy.shape[-1]: {energy.shape[-1]} != len(text)"
+                energy_emb = self.energy_emb(energy)
+            else:
+                energy_pred = self.energy_predictor(prosody_input, enc_mask).squeeze(-1)
+                energy_emb = self.energy_emb(energy_pred.unsqueeze(1))
+            enc_out = enc_out + energy_emb.transpose(1, 2)
         enc_out = enc_out + pitch_emb.transpose(1, 2)
 
         # Expand to decoder time dimension
@@ -304,7 +375,10 @@ class FastPitchModule(NeuralModule):
             volume_extended = volume_extended.squeeze(-1).float()
 
         # Output FFT
-        dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
+        if self.speaker_emb_condition_decoder:
+            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens, conditioning=spk_emb)
+        else:
+            dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
         spect = self.proj(dec_out).transpose(1, 2)
         return (
             spect.to(torch.float),
@@ -314,3 +388,104 @@ class FastPitchModule(NeuralModule):
             pitch_predicted,
             volume_extended,
         )
+
+
+class FastPitchSSLModule(NeuralModule):
+    def __init__(
+        self,
+        encoder_module: NeuralModule,
+        decoder_module: NeuralModule,
+        duration_predictor: NeuralModule,
+        pitch_predictor: NeuralModule,
+        symbols_embedding_dim: int,
+        pitch_embedding_kernel_size: int,
+        n_mel_channels: int = 80,
+        max_token_duration: int = 75,
+    ):
+        super().__init__()
+
+        self.encoder = encoder_module
+        self.decoder = decoder_module
+        self.duration_predictor = duration_predictor
+        self.pitch_predictor = pitch_predictor
+
+        self.max_token_duration = max_token_duration
+        self.min_token_duration = 0
+
+        if self.pitch_predictor is not None:
+            self.pitch_emb = torch.nn.Conv1d(
+                1,
+                symbols_embedding_dim,
+                kernel_size=pitch_embedding_kernel_size,
+                padding=int((pitch_embedding_kernel_size - 1) / 2),
+            )
+
+        # Store values precomputed from training data for convenience
+        self.register_buffer('pitch_mean', torch.zeros(1))
+        self.register_buffer('pitch_std', torch.zeros(1))
+
+        self.proj = torch.nn.Linear(self.decoder.d_model, n_mel_channels, bias=True)
+
+    @property
+    def input_types(self):
+        return {
+            "enc_out": NeuralType(('B', 'T', 'D'), EncodedRepresentation()),
+            "enc_mask": NeuralType(('B', 'T', 'D'), EncodedRepresentation()),
+            "durs": NeuralType(('B', 'T_text'), TokenDurationType(), optional=True),
+            "pitch": NeuralType(('B', 'T_audio'), RegressionValuesType(), optional=True),
+            "pace": NeuralType(optional=True),
+        }
+
+    @property
+    def output_types(self):
+        return {
+            "spect": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType()),
+            "num_frames": NeuralType(('B'), TokenDurationType()),
+            "durs_predicted": NeuralType(('B', 'T_text'), TokenDurationType()),
+            "log_durs_predicted": NeuralType(('B', 'T_text'), TokenLogDurationType()),
+            "pitch_predicted": NeuralType(('B', 'T_text'), RegressionValuesType()),
+            "pitch": NeuralType(('B', 'T_audio'), RegressionValuesType()),
+        }
+
+    @typecheck()
+    def forward(self, *, enc_out=None, enc_mask=None, durs=None, pitch=None, pace=1.0):
+
+        log_durs_predicted, durs_predicted = None, None
+        if self.duration_predictor is not None:
+            log_durs_predicted = self.duration_predictor(enc_out, enc_mask)
+            durs_predicted = torch.clamp(torch.exp(log_durs_predicted) - 1, 0, self.max_token_duration)
+
+        # Predict pitch
+        pitch_predicted = None
+        if self.pitch_predictor is not None:
+            pitch_predicted = self.pitch_predictor(enc_out, enc_mask)
+            if pitch is not None:
+                if pitch.shape[-1] != enc_out.shape[1]:
+                    # during inference, we send the averaged pitch over each token so we don't need to average here
+                    # TODO: have a flag to indicate whether the pitch is already averaged or not
+                    pitch = average_features(pitch.unsqueeze(1), durs).squeeze(1)
+
+                pitch_emb = self.pitch_emb(pitch.unsqueeze(1))
+            else:
+                pitch_emb = self.pitch_emb(pitch_predicted.unsqueeze(1))
+
+            enc_out = enc_out + pitch_emb.transpose(1, 2)
+
+        if durs is not None:
+            len_regulated, dec_lens = regulate_len(durs, enc_out, pace)
+        else:
+            # Use predictions during inference
+            assert self.duration_predictor is not None, "Duration predictor cannot be none if durs is not provided"
+            len_regulated, dec_lens = regulate_len(durs_predicted, enc_out, pace)
+
+        # Output FFT
+        dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
+        spect = self.proj(dec_out).transpose(1, 2)
+        return (
+            spect,
+            dec_lens,
+            durs_predicted,
+            log_durs_predicted,
+            pitch_predicted,
+            pitch,
+        )
