@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import copy
 import inspect
@@ -19,7 +20,7 @@ import uuid
 from abc import abstractmethod
 from os import path
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Union
 
 import hydra
 import torch
@@ -35,6 +36,7 @@ from nemo.core.optim import prepare_lr_scheduler
 from nemo.utils import logging, model_utils
 from nemo.utils.app_state import AppState
 from nemo.utils.debug_hook import register_debug_hooks
+from nemo.utils.exceptions import NeMoBaseException
 from nemo.utils.get_rank import get_rank, is_global_rank_zero
 
 __all__ = ['ModelPT']
@@ -62,7 +64,7 @@ class ModelPT(LightningModule, Model):
         """
         if trainer is not None and not isinstance(trainer, Trainer):
             raise ValueError(
-                f"trainer constructor argument must be either None or pytroch_lightning.Trainer. But got {type(trainer)} instead."
+                f"trainer constructor argument must be either None or pytorch_lightning.Trainer. But got {type(trainer)} instead."
             )
         super().__init__()
 
@@ -110,6 +112,9 @@ class ModelPT(LightningModule, Model):
 
         self._cfg = cfg
 
+        # init mapping submodule attribute -> config_field for nested NeMo models
+        self._nemo_submodule_name_to_config_field = dict()
+
         self.save_hyperparameters("cfg")
         self._train_dl = None
         self._validation_dl = None
@@ -128,13 +133,27 @@ class ModelPT(LightningModule, Model):
             app_state.device_id = torch.cuda.current_device()
 
         if self._cfg is not None and not self._is_model_being_restored():
-            if 'train_ds' in self._cfg and self._cfg.train_ds is not None:
+            # Setup data loaders now (default) or defer setup to `self.setup()`
+            # if `defer_setup` is set in the config of the corresponding dataloader.
+            if (
+                'train_ds' in self._cfg
+                and self._cfg.train_ds is not None
+                and not self._cfg.train_ds.get('defer_setup', False)
+            ):
                 self.setup_training_data(self._cfg.train_ds)
 
-            if 'validation_ds' in self._cfg and self._cfg.validation_ds is not None:
+            if (
+                'validation_ds' in self._cfg
+                and self._cfg.validation_ds is not None
+                and not self._cfg.validation_ds.get('defer_setup', False)
+            ):
                 self.setup_multiple_validation_data(val_data_config=cfg.validation_ds)
 
-            if 'test_ds' in self._cfg and self._cfg.test_ds is not None:
+            if (
+                'test_ds' in self._cfg
+                and self._cfg.test_ds is not None
+                and not self._cfg.test_ds.get('defer_setup', False)
+            ):
                 self.setup_multiple_test_data(test_data_config=cfg.test_ds)
 
         else:
@@ -179,34 +198,43 @@ class ModelPT(LightningModule, Model):
             when model.save_to("mymodel.nemo") is called.
 
             How it works:
+
             1. It always returns existing absolute path which can be used during Model constructor call
                 EXCEPTION: src is None or "" in which case nothing will be done and src will be returned
             2. It will add (config_path, model_utils.ArtifactItem()) pair to self.artifacts
 
-            If "src" is local existing path, then it will be returned in absolute path form.
-            elif "src" starts with "nemo_file:unique_artifact_name":
-                .nemo will be untarred to a temporary folder location and an actual existing path will be returned
-            else an error will be raised.
+                .. code-block::
+
+                    If "src" is local existing path:
+                        then it will be returned in absolute path form.
+                    elif "src" starts with "nemo_file:unique_artifact_name":
+                        .nemo will be untarred to a temporary folder location and an actual existing path will be returned
+                    else:
+                        an error will be raised.
 
             WARNING: use .register_artifact calls in your models' constructors.
-            The returned path is not guaranteed to exist after you have exited your model's constuctor.
+            The returned path is not guaranteed to exist after you have exited your model's constructor.
 
             Args:
                 config_path (str): Artifact key. Usually corresponds to the model config.
                 src (str): Path to artifact.
                 verify_src_exists (bool): If set to False, then the artifact is optional and register_artifact will return None even if
                                           src is not found. Defaults to True.
-                save_restore_connector (SaveRestoreConnector): Can be overrided to add custom save and restore logic.
+                save_restore_connector (SaveRestoreConnector): Can be overridden to add custom save and restore logic.
 
             Returns:
-                str: If src is not None or empty it always returns absolute path which is guaranteed to exists during model instnce life
+                str: If src is not None or empty it always returns absolute path which is guaranteed to exist during model instance life
         """
 
-        app_state = AppState()
-
         if src is None or src == "":
             return src
 
+        if Path(src).suffix == ".nemo":
+            raise NeMoBaseException(
+                "Registering .nemo files as artifacts not supported. "
+                "If you are trying to make a nested model, use `register_nemo_submodule`."
+            )
+
         if not hasattr(self, 'artifacts'):
             self.artifacts = {}
 
@@ -221,6 +249,101 @@ class ModelPT(LightningModule, Model):
 
         return self._save_restore_connector.register_artifact(self, config_path, src, verify_src_exists)
 
+    def has_artifacts(self) -> bool:
+        """Returns True if model has artifacts registered"""
+        return hasattr(self, 'artifacts') and self.artifacts is not None and len(self.artifacts) > 0
+
+    def has_native_or_submodules_artifacts(self) -> bool:
+        """Returns True if it has artifacts or any of the submodules have artifacts"""
+        for module in self.modules():
+            if (
+                isinstance(module, ModelPT)
+                and hasattr(module, 'artifacts')
+                and module.artifacts is not None
+                and len(module.artifacts) > 0
+            ):
+                return True
+        return False
+
+    def has_nemo_submodules(self) -> bool:
+        """Returns True if it has any registered NeMo submodules"""
+        return len(self._nemo_submodule_name_to_config_field) > 0
+
+    def register_nemo_submodule(self, name: str, config_field: str, model: "ModelPT") -> None:
+        """
+        Adds a NeMo model as a submodule. Submodule can be accessed via the `name` attribute on the parent NeMo model this submodule was registered on (`self`).
+        In the saving process, the whole parent model (self) is held as a solid model with artifacts
+        from the child submodule, the submodule config will be saved to the `config_field` of the parent model.
+        This method is necessary to create a nested model, e.g.
+            .. code-block:: python
+
+                class ParentModel(ModelPT):
+                    def __init__(self, cfg, trainer=None):
+                        super().__init__(cfg=cfg, trainer=trainer)
+
+                        # annotate type for autocompletion and type checking (optional)
+                        self.child_model: Optional[ChildModel] = None
+                        if cfg.get("child_model") is not None:
+                            self.register_nemo_submodule(
+                                name="child_model",
+                                config_field="child_model",
+                                model=ChildModel(self.cfg.child_model, trainer=trainer),
+                            )
+                        # ... other code
+
+        Args:
+            name: name of the attribute for the submodule
+            config_field: field in config, where submodule config should be saved
+            model: NeMo model, instance of ModelPT
+        """
+        # check it is a real NeMo model
+        if not isinstance(model, ModelPT):
+            raise NeMoBaseException(
+                f"Model is not and instance of ModelPT, so can't be registered. Got {type(model).__name__}"
+            )
+        # check if it is called after __init__
+        if not hasattr(self, "_nemo_submodule_name_to_config_field"):
+            raise NeMoBaseException(
+                "You are trying to register a submodule before the model is initialized. This is not allowed. "
+                "Did you forget to call `super().__init__`?"
+            )
+        # assign attribute to self
+        setattr(self, name, model)
+        # add to the submodules mapping
+        self._nemo_submodule_name_to_config_field[name] = config_field
+
+    def named_nemo_modules(
+        self, prefix_name: str = "", prefix_config: str = ""
+    ) -> Iterator[Tuple[str, str, "ModelPT"]]:
+        """
+        Returns an iterator over all NeMo submodules recursively, yielding
+        tuples of (attribute path, path in config, submodule), starting from the core module
+
+        Args:
+            prefix_name: prefix for the name path
+            prefix_config: prefix for the path in config
+
+        Returns:
+            Iterator over (attribute path, path in config, submodule), starting from (prefix, self)
+        """
+        if not hasattr(self, "_nemo_submodule_name_to_config_field"):
+            raise NeMoBaseException(
+                "Model is not fully initialized. Calling `named_nemo_modules` before __init__ not allowed. "
+                "Did you forget to call `super().__init__`?"
+            )
+
+        yield prefix_name, prefix_config, self
+
+        # recursive iteration over all NeMo submodules
+        for name, config_field in self._nemo_submodule_name_to_config_field.items():
+            attribute_path = f"{prefix_name}.{name}" if prefix_name else name
+            config_path = f"{prefix_config}.{config_field}" if prefix_config else config_field
+            module: ModelPT = getattr(self, name)
+            for submodule_name, subconfig_path, submodule in module.named_nemo_modules(
+                prefix_name=attribute_path, prefix_config=config_path
+            ):
+                yield submodule_name, subconfig_path, submodule
+
     def save_to(self, save_path: str):
         """
         Saves model instance (weights and configuration) into .nemo file
@@ -611,15 +734,18 @@ class ModelPT(LightningModule, Model):
         """
             Used to create param groups for the optimizer.
             As an example, this can be used to specify per-layer learning rates:
+
             optim.SGD([
                         {'params': model.base.parameters()},
                         {'params': model.classifier.parameters(), 'lr': 1e-3}
                         ], lr=1e-2, momentum=0.9)
+
             See https://pytorch.org/docs/stable/optim.html for more information.
             By default, ModelPT will use self.parameters().
             Override this method to add custom param groups.
             In the config file, add 'optim_param_groups' to support different LRs
             for different components (unspecified params will use the default LR):
+
             model:
                 optim_param_groups:
                     encoder:
@@ -676,6 +802,40 @@ class ModelPT(LightningModule, Model):
         else:
             return [self._optimizer], [self._scheduler]
 
+    def setup(self, stage: Optional[str] = None):
+        """Called at the beginning of fit, validate, test, or predict.
+        This is called on every process when using DDP.
+
+        Args:
+            stage: fit, validate, test or predict
+        """
+        if stage == 'fit':
+            train_deferred_setup = (
+                'train_ds' in self._cfg
+                and self._cfg.train_ds is not None
+                and self._cfg.train_ds.get('defer_setup', False)
+            )
+            if self.train_dataloader() is None and train_deferred_setup:
+                self.setup_training_data(self._cfg.train_ds)
+
+        if stage in ('fit', 'validate'):
+            val_deferred_setup = (
+                'validation_ds' in self._cfg
+                and self._cfg.validation_ds is not None
+                and self._cfg.validation_ds.get('defer_setup', False)
+            )
+            if self.val_dataloader() is None and val_deferred_setup:
+                self.setup_multiple_validation_data(val_data_config=self._cfg.validation_ds)
+
+        if stage == 'test':
+            test_deferred_setup = (
+                'test_ds' in self._cfg
+                and self._cfg.test_ds is not None
+                and self._cfg.test_ds.get('defer_setup', False)
+            )
+            if self.test_dataloader() is None and test_deferred_setup:
+                self.setup_multiple_test_data(test_data_config=self._cfg.test_ds)
+
     def train_dataloader(self):
         if self._train_dl is not None:
             return self._train_dl
@@ -1104,7 +1264,7 @@ class ModelPT(LightningModule, Model):
 
                     # Restore checkpoint into current model
                     self.load_state_dict(restored_model.state_dict(), strict=False)
-                    logging.info(f'Model checkpoint restored from pretrained chackpoint with name : `{model_name}`')
+                    logging.info(f'Model checkpoint restored from pretrained checkpoint with name : `{model_name}`')
 
                     del restored_model
                 elif isinstance(cfg.init_from_pretrained_model, (DictConfig, dict)):
@@ -1140,7 +1300,7 @@ class ModelPT(LightningModule, Model):
                     # Restore checkpoint into current model
                     self.load_state_dict(ckpt['state_dict'], strict=False)
                     logging.info(
-                        f'Model checkpoint restored from pytorch lightning chackpoint with path : `{ckpt_path}`'
+                        f'Model checkpoint restored from pytorch lightning checkpoint with path : `{ckpt_path}`'
                     )
 
                     del ckpt
@@ -1498,7 +1658,7 @@ class ModelPT(LightningModule, Model):
 
             Here we are overriding this to maintain the default Pytorch nn.module behavior:
             https://github.com/pytorch/pytorch/blob/master/torch/nn/modules/module.py#L728
-        
+
         Moves all model parameters and buffers to the GPU.
 
         This also makes associated parameters and buffers different objects. So
