@@ -108,9 +108,12 @@ def cleanup_local_folder():
         rmtree('./nemo_experiments', ignore_errors=True)
 
 
-@pytest.fixture
+@pytest.fixture(scope="session")
 def test_data_dir():
-    """ Fixture returns test_data_dir. """
+    """
+    Fixture returns test_data_dir.
+    Use the highest fixture scope `session` to allow other fixtures with any other scope to use it.
+    """
     # Test dir.
     test_data_dir_ = join(dirname(__file__), __TEST_DATA_SUBDIR)
     return test_data_dir_
@@ -156,6 +159,9 @@ def pytest_configure(config):
     config.addinivalue_line(
         "markers", "run_only_on(device): runs the test only on a given device [CPU | GPU]",
     )
+    config.addinivalue_line(
+        "markers", "with_downloads: runs the test using data present in tests/.data",
+    )
     # Test dir and archive filepath.
     test_dir = join(dirname(__file__), __TEST_DATA_SUBDIR)
     test_data_archive = join(dirname(__file__), __TEST_DATA_SUBDIR, __TEST_DATA_FILENAME)
@@ -227,7 +233,7 @@ def pytest_configure(config):
         numba_utils.set_numba_compat_strictness(strict=config.option.relax_numba_compat)
 
     # Set cache directory for TN/ITN tests
-    from .nemo_text_processing.utils import set_cache_dir, set_audio_based_tests
+    from .nemo_text_processing.utils import set_audio_based_tests, set_cache_dir
 
     set_cache_dir(config.option.tn_cache_dir)
     set_audio_based_tests(config.option.run_audio_based)
