@@ -20,6 +20,7 @@ import pytest
 import pytorch_lightning as pl
 import torch
 import torch.optim
+from pytorch_lightning.utilities import rank_zero_only
 
 from nemo.core import config, optim
 from nemo.core.optim.lr_scheduler import AVAILABLE_SCHEDULERS
@@ -85,7 +86,7 @@ class ExampleModel(pl.LightningModule):
 
 
 class Callback(pl.callbacks.Callback):
-    @pl.utilities.distributed.rank_zero_only
+    @rank_zero_only
     def on_train_end(self, trainer, module):
         count = module.my_opt.param_groups[0]['count']
         if trainer.global_step != count or trainer.global_step != module.max_steps:
@@ -110,13 +111,13 @@ class SchedulerNoOpCallback(Callback):
     def on_train_batch_end(self, trainer: pl.Trainer, pl_module, outputs, batch, batch_idx):
         # pl_module.max_steps is "original" max steps without trainer extra steps.
         if (trainer.global_step + 1) % 3 == 0 and (trainer.global_step + 1) < pl_module.max_steps:
-            schedulers = trainer.lr_schedulers
+            schedulers = trainer.lr_scheduler_configs
 
             for scheduler in schedulers:
                 # Decrement the counter by 2, then perform a scheduler.step() to perform a no-up
                 # as well as update the optimizer lr in all param groups
-                scheduler['scheduler'].last_epoch -= 2
-                scheduler['scheduler'].step()
+                scheduler.scheduler.last_epoch -= 2
+                scheduler.scheduler.step()
 
             # Increase the max step count by 1
             trainer.fit_loop.max_steps = trainer.fit_loop.max_steps + 1
@@ -309,6 +310,51 @@ class TestOptimizersSchedulers:
         scheduler_setup = optim.lr_scheduler.prepare_lr_scheduler(opt, dict_config)
         assert isinstance(scheduler_setup['scheduler'], optim.lr_scheduler.CosineAnnealing)
 
+    @pytest.mark.unit
+    def test_sched_config_parse_reduce_on_plateau(self):
+        model = TempModel()
+        opt_cls = optim.get_optimizer('novograd')
+        opt = opt_cls(model.parameters(), lr=self.INITIAL_LR)
+        reduce_on_plateau_parameters = {
+            'mode': 'min',
+            'factor': 0.5,
+            'patience': 1,
+            'threshold': 1e-4,
+            'threshold_mode': 'rel',
+            'min_lr': 1e-6,
+            'eps': 1e-7,
+            'verbose': True,
+            'cooldown': 1,
+        }
+        basic_sched_config = {
+            'name': 'ReduceLROnPlateau',
+            'monitor': 'val_loss',
+            'reduce_on_plateau': True,
+            'max_steps': self.MAX_STEPS,
+        }
+        basic_sched_config.update(reduce_on_plateau_parameters)
+        scheduler_setup = optim.lr_scheduler.prepare_lr_scheduler(opt, basic_sched_config)
+        assert isinstance(scheduler_setup['scheduler'], torch.optim.lr_scheduler.ReduceLROnPlateau)
+        for k, v in reduce_on_plateau_parameters.items():
+            if k == 'min_lr':
+                k += 's'
+                v = [v]
+            found_v = getattr(scheduler_setup['scheduler'], k)
+            assert (
+                found_v == v
+            ), f"Wrong value `{repr(found_v)}` for `ReduceLROnPlateau` parameter `{k}`. Expected `{repr(v)}`."
+        dict_config = omegaconf.OmegaConf.create(basic_sched_config)
+        scheduler_setup = optim.lr_scheduler.prepare_lr_scheduler(opt, dict_config)
+        assert isinstance(scheduler_setup['scheduler'], torch.optim.lr_scheduler.ReduceLROnPlateau)
+        for k, v in reduce_on_plateau_parameters.items():
+            if k == 'min_lr':
+                k += 's'
+                v = [v]
+            found_v = getattr(scheduler_setup['scheduler'], k)
+            assert (
+                found_v == v
+            ), f"Wrong value `{repr(found_v)}` for `ReduceLROnPlateau` parameter `{k}`. Expected `{repr(v)}`."
+
     @pytest.mark.unit
     def test_WarmupPolicy(self):
         model = TempModel()
